/*
  tsemgr is a gtk application to manage a sonyericsson t68 mobile phone
  Copyright (C) 2003 Sebastian Sareyko

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

#include "config.h"
#include <ezV24/ezV24.h>
#include <string.h>
#include <ctype.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <errno.h>
#ifdef HAVE_BLUETOOTH
#include <sys/socket.h>
#include <arpa/inet.h>
#include <netdb.h>
#include <netinet/in.h>
#include <bluetooth/bluetooth.h>
#include <bluetooth/rfcomm.h>
#endif
#include <openobex/obex.h>
#include <signal.h>
#include "callbacks.h"
#include "phonebook.h"
#include "tsemgr.h"
#include "interface.h"
#include "obex_misc.h"
#include "putfile.h"
#include "getfile.h"
#include "sms.h"
#include "pixmaps.h"
#include "remote.h"
#include "gsm.h"
#include "cobex.h"
#include <iconv.h>

void save_config (GtkWidget *button, gpointer user_data) {
  GtkWidget *entry;
  gchar *tmp;
  int fd;

  entry = gtk_object_get_data (GTK_OBJECT (button), "device");
  config.device = gtk_entry_get_text (GTK_ENTRY (entry));
#ifdef HAVE_BLUETOOTH
  entry = gtk_object_get_data (GTK_OBJECT (button), "address");
  config.bdaddress = gtk_entry_get_text (GTK_ENTRY (entry));
  entry = gtk_object_get_data (GTK_OBJECT (button), "chan");
  config.opchan = gtk_entry_get_text (GTK_ENTRY (entry));
#endif
  if (config_base_dir != NULL) {
    tmp = g_strdup_printf ("%s/%s", config_base_dir, BASE_CONFIG_FILE);
    fd = open (tmp, O_RDWR | O_CREAT | O_TRUNC, 0666);
    g_free (tmp);

    if (fd != -1) {
      tmp = g_strdup_printf ("%s\n", config.device);
#ifdef HAVE_BLUETOOTH
      tmp = g_strdup_printf ("%s%s\n%s\n", tmp, config.bdaddress, config.opchan);
#endif
      if (write (fd, tmp, sizeof (gchar) * strlen (tmp)) == -1) {
	errorDialog ("cannot write to file!");
      }
      close (fd);
    }
    else
      errorDialog ("cannot save to file!");
  }
  else
    errorDialog ("cannot save to file!!\nreason:\nyour $HOME variable is not set\ndo export HOME=/pathto/yourhomedir\nand start tsemgr again");
}

void backupPB (GtkWidget *button, gpointer user_data) {
  GtkWidget *radio;
  GtkWidget *dialog;
  obex_t *handle;
  struct context global_context = {0,};
  obex_ctrans_t custfunc;
#ifdef HAVE_BLUETOOTH
  bdaddr_t bdaddr;
  uint8_t channel;
#endif

  if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (user_data)) == TRUE) {
    if ((handle = OBEX_Init (OBEX_TRANS_IRDA, obex_event, 0))) {
      OBEX_SetUserData (handle, &global_context);
      if (IrOBEX_TransportConnect (handle, "OBEX") >= 0) {
	if (t68_connect (handle) == 0) {
	  dialog = messageDialog ("backing up your phonebook...\n", 0);
	  gtk_widget_show_all (dialog);
	  getFile (handle, &global_context, "telecom/pb.vcf");
	  t68_disconnect (handle);
	  gtk_widget_destroy (dialog);
	}
	else
	  errorDialog ("can not connect to phone!");
      }
      else
	errorDialog ("transport connect error!");
    }
  }
  else {
    radio = gtk_object_get_data (GTK_OBJECT (user_data), "radio_pbbt");
    if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (radio)) == TRUE) {
#ifdef HAVE_BLUETOOTH
      if ((handle = OBEX_Init (OBEX_TRANS_BLUETOOTH, obex_event, 0))) {
	OBEX_SetUserData (handle, &global_context);
	str2ba (config.bdaddress, &bdaddr);
	channel = (uint8_t) atoi (config.opchan);
	if (BtOBEX_TransportConnect (handle, BDADDR_ANY, &bdaddr, channel) >= 0) {
	  if (t68_connect (handle) == 0) {
	    dialog = messageDialog ("backing up your phonebook...\n", 0);
	    gtk_widget_show_all (dialog);
	    getFile (handle, &global_context, "telecom/pb.vcf");
	    t68_disconnect (handle);
	    gtk_widget_destroy (dialog);
	  }
	  else
	    errorDialog ("can not connect to phone!");
	}
	else
	  errorDialog ("Transport connect error!");
      }
#else
      errorDialog ("your version of tsemgr is\nNOT\ncompiled with bluetooth support!\n\nsee README for details");
#endif
    }
    else {
      custfunc.customdata = cobex_open (config.device);
      if ((handle = OBEX_Init (OBEX_TRANS_CUSTOM, obex_event, 0))) {
	custfunc.connect = cobex_connect;
	custfunc.disconnect = cobex_disconnect;
	custfunc.write = cobex_write;
	custfunc.handleinput = cobex_handle_input;
	custfunc.listen = cobex_connect; /* Listen and connect is
					    100% same on cable */
	OBEX_RegisterCTransport (handle, &custfunc);
	OBEX_SetUserData (handle, &global_context);
	if (OBEX_TransportConnect (handle, (void*) 1, 0) >= 0) {
	  if (t68_connect (handle) == 0) {
	    dialog = messageDialog ("backing up your phonebook...\n", 0);
	    gtk_widget_show_all (dialog);
	    getFile (handle, &global_context, "telecom/pb.vcf");
	    t68_disconnect (handle);
	    gtk_widget_destroy (dialog);
	  }
	  else
	    errorDialog ("can not connect to phone!");
	}
	else
	  errorDialog ("Transport connect error!");
      }
    }
  }
}

void restorePB (GtkWidget *button, gpointer user_data) {
  GtkWidget *radio;
  GtkWidget *dialog;
  gchar *filename, *errmsg, *name;
  struct stat mystat;
  obex_t *handle;
  struct context global_context = {0,};
  obex_ctrans_t custfunc;
#ifdef HAVE_BLUETOOTH
  bdaddr_t bdaddr;
  uint8_t channel;
#endif

  filename = g_strdup_printf ("%s/pb.vcf", config_base_dir);
  name = g_strdup ("telecom/pb.vcf");

  if (stat (filename, &mystat) == 0) {
    if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (user_data)) == TRUE) {
      if ((handle = OBEX_Init (OBEX_TRANS_IRDA, obex_event, 0))) {
	OBEX_SetUserData (handle, &global_context);
	if (IrOBEX_TransportConnect (handle, "OBEX") >= 0) {
	  if (t68_connect (handle) == 0) {
	    dialog = messageDialog ("restoring your whole phonebook...\n", 0);
	    gtk_widget_show_all (dialog);
	    sendFile (handle, filename, name);
	    t68_disconnect (handle);
	    gtk_widget_destroy (dialog);
	  }
	  else
	    errorDialog ("can not connect to phone!");
	}
	else
	  errorDialog ("transport connect error!");
      }
    }
    else {
      radio = gtk_object_get_data (GTK_OBJECT (user_data), "radio_pbbt");
      if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (radio)) == TRUE) {
#ifdef HAVE_BLUETOOTH
	if ((handle = OBEX_Init (OBEX_TRANS_BLUETOOTH, obex_event, 0))) {
	  OBEX_SetUserData (handle, &global_context);
	  str2ba (config.bdaddress, &bdaddr);
	  channel = (uint8_t) atoi (config.opchan);
	  if (BtOBEX_TransportConnect (handle, BDADDR_ANY, &bdaddr, channel) >= 0) {
	    if (t68_connect (handle) == 0) {
	      dialog = messageDialog ("restoring your whole phonebook...\n", 0);
	      gtk_widget_show_all (dialog);
	      sendFile (handle, filename, name);
	      t68_disconnect (handle);
	      gtk_widget_destroy (dialog);
	    }
	    else
	      errorDialog ("can not connect to phone!");
	  }
	  else
	    errorDialog ("Transport connect error!");
	}
#else
	errorDialog ("your version of tsemgr is\nNOT\ncompiled with bluetooth support!\n\nsee README for details");
#endif
      }
      else {
	custfunc.customdata = cobex_open (config.device);
	if ((handle = OBEX_Init (OBEX_TRANS_CUSTOM, obex_event, 0))) {
	  custfunc.connect = cobex_connect;
	  custfunc.disconnect = cobex_disconnect;
	  custfunc.write = cobex_write;
	  custfunc.handleinput = cobex_handle_input;
	  custfunc.listen = cobex_connect; /* Listen and connect is
					      100% same on cable */
	  OBEX_RegisterCTransport (handle, &custfunc);
	  OBEX_SetUserData (handle, &global_context);
	  if (OBEX_TransportConnect (handle, (void*) 1, 0) >= 0) {
	    if (t68_connect (handle) == 0) {
	      dialog = messageDialog ("restoring your whole phonebook...\n", 0);
	      gtk_widget_show_all (dialog);
	      sendFile (handle, filename, name);
	      t68_disconnect (handle);
	      gtk_widget_destroy (dialog);
	    }
	    else
	      errorDialog ("can not connect to phone!");
	  }
	  else
	    errorDialog ("Transport connect error!");
	}
      }
    }
  }
  else {
    errmsg = g_strdup_printf ("file not found:\n%s\ndid you backed up your pb before?", filename);
    errorDialog (errmsg);
    g_free (errmsg);
  }
  //  g_free (filename);
}

void restorePB_ask (GtkWidget *button, gpointer user_data) {
  GtkWidget *dialog, *label;
  GtkWidget *okay_button, *cancel_button;
   
  dialog = gtk_dialog_new ();
  label = gtk_label_new ("are you sure you want to\nrestore your phonebook?\nALL current phonebook\nentries on your phone will get lost!!");
  okay_button = gtk_button_new_with_label ("Yes");
  cancel_button = gtk_button_new_with_label ("No");
   
  gtk_signal_connect (GTK_OBJECT (okay_button), "clicked",
		      GTK_SIGNAL_FUNC (restorePB), user_data);
  gtk_signal_connect_object (GTK_OBJECT (okay_button), "clicked",
			     GTK_SIGNAL_FUNC (gtk_widget_destroy),
			     (gpointer) dialog);

  gtk_signal_connect_object (GTK_OBJECT (cancel_button), "clicked",
			     GTK_SIGNAL_FUNC (gtk_widget_destroy),
			     (gpointer) dialog);
  gtk_container_add (GTK_CONTAINER (GTK_DIALOG(dialog)->action_area),
		     okay_button);
  gtk_container_add (GTK_CONTAINER (GTK_DIALOG(dialog)->action_area),
		     cancel_button);

  gtk_container_add (GTK_CONTAINER (GTK_DIALOG(dialog)->vbox),
		     label);
  gtk_window_set_modal (GTK_WINDOW (dialog), TRUE);
  gtk_widget_show_all (dialog);
}

gint pbcomparefunc (GtkCList *clist, gpointer p1, gpointer p2) {
  gchar *entry1, *entry2;
  GtkCListRow *row1 = (GtkCListRow *) p1;
  GtkCListRow *row2 = (GtkCListRow *) p2;

  switch (row1->cell[clist->sort_column].type) {
  case GTK_CELL_TEXT:
    entry1 = GTK_CELL_TEXT (row1->cell[clist->sort_column])->text;
    break;
  case GTK_CELL_PIXTEXT:
    entry1 = GTK_CELL_PIXTEXT (row1->cell[clist->sort_column])->text;
    break;
  default:
    break;
  }
  switch (row2->cell[clist->sort_column].type) {
  case GTK_CELL_TEXT:
    entry2 = GTK_CELL_TEXT (row2->cell[clist->sort_column])->text;
    break;
  case GTK_CELL_PIXTEXT:
    entry2 = GTK_CELL_PIXTEXT (row2->cell[clist->sort_column])->text;
    break;
  default:
    break;
  }

  if (!entry2)
    return 1;
  if (!entry1)
    return -1;
  if (clist->sort_column != 0)
    return g_strcasecmp (entry1, entry2);
  else
    return (atoi (entry1) < atoi (entry2)) ? -1 : 1;
}

void pbsortfunc (GtkCList *clist, gint column, gpointer user_data) {
  if(clist->sort_type == GTK_SORT_ASCENDING)
    clist->sort_type = GTK_SORT_DESCENDING;
  else
    clist->sort_type = GTK_SORT_ASCENDING;
  gtk_clist_set_sort_column (GTK_CLIST (clist), column);
  gtk_clist_set_compare_func (GTK_CLIST (clist),
			      (GtkCListCompareFunc) pbcomparefunc);
  gtk_clist_sort (GTK_CLIST (clist));
}

void save_remote_list (GtkWidget *button, gpointer user_data) {
  GtkCTreeNode *node, *parent, *root;
  gchar *tmp;
  gchar *nodestr_parent, *nodestr1, *nodestr2;
  gchar *to_save;
  guint8 spacing;
  GdkPixmap *pixmap;
  GdkBitmap *mask;
  gint nodetype;
  int fd;

  if (config_base_dir != NULL) {
    tmp = g_strdup_printf ("%s/%s", config_base_dir, REMOTE_CTREE_FILE);
    fd = open (tmp, O_RDWR | O_CREAT | O_TRUNC, 0666);
    g_free (tmp);

    if (fd != -1) {
      to_save = g_strdup ("");
      root = gtk_object_get_data (GTK_OBJECT (user_data), "root");
      gtk_ctree_expand_recursive (GTK_CTREE (user_data), NULL);
      node = GTK_CTREE_NODE_NEXT (root);
      while (node != NULL) {
	parent = GTK_CTREE_ROW (node)->parent;
	nodetype = (gint) gtk_ctree_node_get_row_data (GTK_CTREE (user_data), node);
	if (nodetype) {
	  gtk_ctree_node_get_text (GTK_CTREE (user_data), node, 1, &tmp);
	  nodestr2 = g_strdup (tmp);
	}
	else
	  nodestr2 = g_strdup (" ");
	if (parent == root)
	  nodestr_parent = g_strdup ("root");
	else {
	  gtk_ctree_node_get_pixtext (GTK_CTREE (user_data), parent, 0, &tmp,
				      &spacing, &pixmap, &mask);
	  nodestr_parent = g_strdup (tmp);
	}
	gtk_ctree_node_get_pixtext (GTK_CTREE (user_data), node, 0, &tmp,
				    &spacing, &pixmap, &mask);
	nodestr1 = g_strdup (tmp);
	to_save = g_strdup_printf ("%s%d,%s,%s,%s\n", to_save, nodetype,
				   nodestr_parent, nodestr1, nodestr2);
	node = GTK_CTREE_NODE_NEXT (node);
	g_free (nodestr1);
	g_free (nodestr2);
	g_free (nodestr_parent);
      }
      if (write (fd, to_save, sizeof (gchar) * strlen (to_save)) == -1) {
	errorDialog ("cannot write to file!");
      }
      close (fd);
    }
    else
      errorDialog ("cannot save to file!");
  }
  else
    errorDialog ("cannot save to file!!\nreason:\nyour $HOME variable is not set\ndo export HOME=/pathto/yourhomedir\nand start tsemgr again");
}

void edit_remote_entry_dialog (GtkWidget *button, gpointer user_data) {
  GtkWidget *dialog, *label, *entry;
  GtkWidget *ok_button, *cancel_button, *table;
  GtkCTreeNode *selnode, *root;
  gint nodetype = 0;
  gchar *temp = NULL;
  guint8 spacing;
  GdkPixmap *pixmap;
  GdkBitmap *mask;

  selnode = GTK_CTREE_NODE (g_list_nth (GTK_CLIST (user_data)->row_list,
					GTK_CLIST (user_data)->focus_row));
  nodetype = (gint) gtk_ctree_node_get_row_data (GTK_CTREE (user_data), selnode);
  root = gtk_object_get_data (GTK_OBJECT (user_data), "root");
  if (selnode != root) {
    dialog = gtk_dialog_new ();
    ok_button = gtk_button_new_with_label ("save");
    cancel_button = gtk_button_new_with_label ("cancel");

    if (nodetype == 1)
      table = gtk_table_new (2, 1, FALSE);
    else
      table = gtk_table_new (2, 2, FALSE);

    label = gtk_label_new ("name:");
    entry = gtk_entry_new ();
    gtk_ctree_node_get_pixtext (GTK_CTREE (user_data), selnode, 0, &temp,
				&spacing, &pixmap, &mask);
    gtk_entry_set_text (GTK_ENTRY (entry), temp);
    gtk_object_set_data (GTK_OBJECT (user_data), "name", entry);

    gtk_table_attach_defaults (GTK_TABLE (table), label, 0, 1, 0, 1);
    gtk_table_attach_defaults (GTK_TABLE (table), entry, 1, 2, 0, 1);

    if (nodetype == 1) {
      label = gtk_label_new ("command:");
      entry = gtk_entry_new ();
      gtk_ctree_node_get_text (GTK_CTREE (user_data), selnode, 1, &temp);
      gtk_entry_set_text (GTK_ENTRY (entry), temp);
      gtk_object_set_data (GTK_OBJECT (user_data), "command", entry);

      gtk_table_attach_defaults (GTK_TABLE (table), label, 0, 1, 1, 2);
      gtk_table_attach_defaults (GTK_TABLE (table), entry, 1, 2, 1, 2);
    }
    gtk_container_add (GTK_CONTAINER (GTK_DIALOG(dialog)->vbox), table);

    gtk_object_set_data (GTK_OBJECT (user_data), "dialog", (gpointer) dialog);
    gtk_signal_connect (GTK_OBJECT (ok_button), "clicked",
			GTK_SIGNAL_FUNC (edit_remote_entry), user_data);

    gtk_signal_connect_object (GTK_OBJECT (cancel_button), "clicked",
			       GTK_SIGNAL_FUNC (gtk_widget_destroy),
			       (gpointer) dialog);

    gtk_container_add (GTK_CONTAINER (GTK_DIALOG(dialog)->action_area),
		       ok_button);
    gtk_container_add (GTK_CONTAINER (GTK_DIALOG(dialog)->action_area),
		       cancel_button);
  
    gtk_window_set_modal (GTK_WINDOW (dialog), TRUE);
    
    gtk_widget_show_all (dialog);
  }
  else
    errorDialog ("you can't edit\nthis entry!");
}

void edit_remote_entry (GtkWidget *button, gpointer user_data) {
  GtkWidget *entry, *dialog;
  GtkCTreeNode *selnode;
  gint nodetype = 0;
  gchar *tmp1, *tmp2;

  selnode = GTK_CTREE_NODE (g_list_nth (GTK_CLIST (user_data)->row_list,
					GTK_CLIST (user_data)->focus_row));
  nodetype = (gint) gtk_ctree_node_get_row_data (GTK_CTREE (user_data), selnode);
  if (nodetype == 1) {
    entry = gtk_object_get_data (GTK_OBJECT (user_data), "command");
    tmp2 = gtk_entry_get_text (GTK_ENTRY (entry));
  }
  else
    tmp2 = g_strdup (" ");

  entry = gtk_object_get_data (GTK_OBJECT (user_data), "name");
  tmp1 = gtk_entry_get_text (GTK_ENTRY (entry));
  if ((strstr (tmp1, ",") == 0) && (strstr (tmp2, ",") == 0)) {
    gtk_ctree_node_set_text (GTK_CTREE (user_data), selnode, 0, tmp1);
    gtk_ctree_node_set_text (GTK_CTREE (user_data), selnode, 1, tmp2);
    dialog = (GtkWidget *) gtk_object_get_data (GTK_OBJECT (user_data), "dialog");
    gtk_widget_destroy (GTK_WIDGET (dialog));
  }
  else
    errorDialog ("please don't use one\nfollowing characters: ,");
}

void delete_remote_entry_dialog (GtkWidget *button, gpointer user_data) {
  GtkWidget *dialog, *label;
  GtkWidget *okay_button, *cancel_button;
  GtkCTreeNode *selnode, *root;

  selnode = GTK_CTREE_NODE (g_list_nth (GTK_CLIST (user_data)->row_list,
					GTK_CLIST (user_data)->focus_row));
  root = gtk_object_get_data (GTK_OBJECT (user_data), "root");
  if (selnode != root) {
    dialog = gtk_dialog_new ();
    label = gtk_label_new ("are you sure you want to\ndelete\n the selected entry?");
    okay_button = gtk_button_new_with_label ("Yes");
    cancel_button = gtk_button_new_with_label ("No");
   
    /* Ensure that the dialog box is destroyed when the user clicks ok. */
   
    gtk_signal_connect (GTK_OBJECT (okay_button), "clicked",
			GTK_SIGNAL_FUNC (delete_remote_entry), user_data);
    gtk_signal_connect_object (GTK_OBJECT (okay_button), "clicked",
			       GTK_SIGNAL_FUNC (gtk_widget_destroy),
			       (gpointer) dialog);

    gtk_signal_connect_object (GTK_OBJECT (cancel_button), "clicked",
			       GTK_SIGNAL_FUNC (gtk_widget_destroy),
			       (gpointer) dialog);
    gtk_container_add (GTK_CONTAINER (GTK_DIALOG(dialog)->action_area),
		       okay_button);
    gtk_container_add (GTK_CONTAINER (GTK_DIALOG(dialog)->action_area),
		       cancel_button);

    /* Add the label, and show everything we've added to the dialog. */

    gtk_container_add (GTK_CONTAINER (GTK_DIALOG(dialog)->vbox),
		       label);
    gtk_window_set_modal (GTK_WINDOW (dialog), TRUE);
    gtk_widget_show_all (dialog);
  }
  else
    errorDialog ("you can't delete\nthis entry!");
}

void delete_remote_entry (GtkWidget *button, gpointer user_data) {
  GtkCTreeNode *selnode;

  selnode = GTK_CTREE_NODE (g_list_nth (GTK_CLIST (user_data)->row_list,
					GTK_CLIST (user_data)->focus_row));
  gtk_ctree_remove_node (GTK_CTREE (user_data), selnode);
}

void remote_connect (GtkWidget *button, gpointer user_data) {
  GtkWidget *dialog, *label;
  GtkWidget *disconnect_button;
  v24_port_t *port = NULL;
  v24_port_t *porttmp = NULL;

  porttmp = gtk_object_get_data (GTK_OBJECT (user_data), "v24port");
  if (!porttmp) {
    dialog = messageDialog ("connecting to phone\nplease wait...", 0);
    while (gtk_events_pending ())
      gtk_main_iteration ();
    port = v24OpenPort (config.device, V24_STANDARD);
    gtk_object_set_data (GTK_OBJECT (user_data), "v24port", (gpointer) port);
    gtk_widget_destroy (dialog);
    if (port != NULL) {
      create_accessory_menus_and_wait (user_data);
      dialog = gtk_dialog_new ();
      gtk_window_set_modal (GTK_WINDOW (dialog), TRUE);
      disconnect_button = gtk_button_new_with_label ("disconnect");
      gtk_signal_connect (GTK_OBJECT (disconnect_button), "clicked",
			  GTK_SIGNAL_FUNC (remote_disconnect), user_data);
      gtk_signal_connect_object (GTK_OBJECT (disconnect_button), "clicked",
				 GTK_SIGNAL_FUNC (gtk_widget_destroy),
				 (gpointer) dialog);
      label = gtk_label_new ("connected!\nselect connections->accessories->linux box\n to use your phone as remote control now.");
      gtk_container_add (GTK_CONTAINER (GTK_DIALOG(dialog)->vbox), label);
      gtk_container_add (GTK_CONTAINER (GTK_DIALOG(dialog)->action_area),
			 disconnect_button);
      gtk_widget_show_all (dialog);
    }
    else
      errorDialog ("unable to open device!\nplease check if your phone is connected\nand if you specified the correct device!");
  }
}

void remote_disconnect (GtkWidget *button, gpointer user_data) {
  v24_port_t *port = NULL;
  pid_t pid;

  port = gtk_object_get_data (GTK_OBJECT (user_data), "v24port");
  pid = (pid_t) gtk_object_get_data (GTK_OBJECT (user_data), "pid");
  if (port) {
    v24ClosePort (port);
    kill ((pid_t) -1 * pid, SIGHUP);
    gtk_object_set_data (GTK_OBJECT (user_data), "v24port", NULL);
  }
}

void add_remote_entry_dialog (GtkWidget *button, gpointer user_data) {
  GtkWidget *dialog, *label, *entry;
  GtkWidget *ok_button, *cancel_button;

  dialog = gtk_dialog_new ();
  ok_button = gtk_button_new_with_label ("add");
  cancel_button = gtk_button_new_with_label ("cancel");

  label = gtk_label_new ("name:");
  entry = gtk_entry_new ();

  gtk_container_add (GTK_CONTAINER (GTK_DIALOG(dialog)->vbox), label);
  gtk_container_add (GTK_CONTAINER (GTK_DIALOG(dialog)->vbox), entry);

  gtk_object_set_data (GTK_OBJECT (user_data), "name", entry);
  gtk_object_set_data (GTK_OBJECT (user_data), "dialog", (gpointer) dialog);

  gtk_signal_connect (GTK_OBJECT (ok_button), "clicked",
		      GTK_SIGNAL_FUNC (add_remote_entry), user_data);

  gtk_signal_connect_object (GTK_OBJECT (cancel_button), "clicked",
			     GTK_SIGNAL_FUNC (gtk_widget_destroy),
			     (gpointer) dialog);

  gtk_container_add (GTK_CONTAINER (GTK_DIALOG(dialog)->action_area),
		     ok_button);
  gtk_container_add (GTK_CONTAINER (GTK_DIALOG(dialog)->action_area),
		     cancel_button);

  gtk_window_set_modal (GTK_WINDOW (dialog), TRUE);

  gtk_widget_show_all (dialog);
}

void add_remote_command_dialog (GtkWidget *button, gpointer user_data) {
  GtkWidget *dialog, *label, *entry;
  GtkWidget *ok_button, *cancel_button, *table;

  dialog = gtk_dialog_new ();
  ok_button = gtk_button_new_with_label ("add");
  cancel_button = gtk_button_new_with_label ("cancel");

  table = gtk_table_new (2, 2, FALSE);

  label = gtk_label_new ("name:");
  entry = gtk_entry_new ();
  gtk_object_set_data (GTK_OBJECT (user_data), "name", entry);

  gtk_table_attach_defaults (GTK_TABLE (table), label, 0, 1, 0, 1);
  gtk_table_attach_defaults (GTK_TABLE (table), entry, 1, 2, 0, 1);

  label = gtk_label_new ("command:");
  entry = gtk_entry_new ();
  gtk_object_set_data (GTK_OBJECT (user_data), "command", entry);

  gtk_table_attach_defaults (GTK_TABLE (table), label, 0, 1, 1, 2);
  gtk_table_attach_defaults (GTK_TABLE (table), entry, 1, 2, 1, 2);

  gtk_container_add (GTK_CONTAINER (GTK_DIALOG(dialog)->vbox), table);
  gtk_object_set_data (GTK_OBJECT (user_data), "dialog", (gpointer) dialog);

  gtk_signal_connect (GTK_OBJECT (ok_button), "clicked",
		      GTK_SIGNAL_FUNC (add_remote_command), user_data);

  gtk_signal_connect_object (GTK_OBJECT (cancel_button), "clicked",
			     GTK_SIGNAL_FUNC (gtk_widget_destroy),
			     (gpointer) dialog);

  gtk_container_add (GTK_CONTAINER (GTK_DIALOG(dialog)->action_area),
		     ok_button);
  gtk_container_add (GTK_CONTAINER (GTK_DIALOG(dialog)->action_area),
		     cancel_button);
  
  gtk_window_set_modal (GTK_WINDOW (dialog), TRUE);
    
  gtk_widget_show_all (dialog);
}

void add_remote_entry (GtkWidget *button, gpointer user_data) {
  GtkWidget *entry, *dialog;
  GtkCTreeNode *root, *node;
  gchar *tmp[2];

  entry = gtk_object_get_data (GTK_OBJECT (user_data), "name");
  tmp[0] = gtk_entry_get_text (GTK_ENTRY (entry));
  if (strstr (tmp[0], ",") == 0) {
    dialog = gtk_object_get_data (GTK_OBJECT (user_data), "dialog");
    tmp[1] = g_strdup (" ");
    root = gtk_object_get_data (GTK_OBJECT (user_data), "root");
    node = gtk_ctree_insert_node (GTK_CTREE (user_data), root, NULL, tmp, 2,
				  NULL, NULL, NULL, NULL, FALSE, FALSE);
    gtk_ctree_node_set_row_data (GTK_CTREE (user_data), node, (gpointer) 0);
    gtk_ctree_expand_recursive (GTK_CTREE (user_data), root);
    gtk_widget_destroy (dialog);
  }
  else
    errorDialog ("please don't use one\nfollowing characters: ,");
}

void add_remote_command (GtkWidget *button, gpointer user_data) {
  GtkWidget *entry, *dialog;
  GtkCTreeNode *node, *selnode, *root;
  gchar *tmp[2];
  gint nodetype = 0;

  entry = gtk_object_get_data (GTK_OBJECT (user_data), "name");
  tmp[0] = gtk_entry_get_text (GTK_ENTRY (entry));
  entry = gtk_object_get_data (GTK_OBJECT (user_data), "command");
  tmp[1] = gtk_entry_get_text (GTK_ENTRY (entry));
  if ((strstr (tmp[0], ",") == 0) && (strstr (tmp[1], ",") == 0)) {
    dialog = gtk_object_get_data (GTK_OBJECT (user_data), "dialog");
    selnode = GTK_CTREE_NODE (g_list_nth (GTK_CLIST (user_data)->row_list,
					  GTK_CLIST (user_data)->focus_row));
    nodetype = (gint) gtk_ctree_node_get_row_data (GTK_CTREE (user_data),
						   selnode);
    if (nodetype == 1)
      selnode = GTK_CTREE_ROW (selnode)->parent;
    root = gtk_object_get_data (GTK_OBJECT (user_data), "root");
    node = gtk_ctree_insert_node (GTK_CTREE (user_data), selnode, NULL, tmp, 2,
				  NULL, NULL, NULL, NULL, FALSE, FALSE);
    gtk_ctree_node_set_row_data (GTK_CTREE (user_data), node, (gpointer) 1);
    gtk_ctree_expand_recursive (GTK_CTREE (user_data), root);
    gtk_widget_destroy (dialog);
  }
  else
    errorDialog ("please don't use one\nfollowing characters: ,");
}

void displaySMSListME (GtkWidget *widget, gpointer *scrolled_window) {
  displaySMSList (widget, scrolled_window, ME);
}

void displaySMSListSIM (GtkWidget *widget, gpointer *scrolled_window) {
  displaySMSList (widget, scrolled_window, SIM);
}

gint timeout (gpointer data) {
  while (gtk_events_pending ())
    gtk_main_iteration ();
  gtk_timeout_add (100, timeout, NULL);
  return FALSE;
}

void filenamechanged (GtkWidget *entry, gpointer user_data) {
  GtkWidget *name_entry;
  gchar *tmp, *filename, *nameonphone;

  name_entry = gtk_object_get_data (GTK_OBJECT (user_data), "nameop");
  //  nameonphone = gtk_editable_get_chars (GTK_EDITABLE (name_entry), 0, -1);
  filename = gtk_editable_get_chars (GTK_EDITABLE (user_data), 0, -1);
  if (strcmp (filename, "") != 0) {
    if ((tmp = rindex (filename, '/')) != NULL) {
      ++tmp;
      nameonphone = g_strdup (tmp);
    }
    else
      nameonphone = g_strdup (filename);
    gtk_entry_set_text (GTK_ENTRY (name_entry), nameonphone);
  }
  else
    gtk_entry_set_text (GTK_ENTRY (name_entry), "");
}

void uploadToPhone (GtkWidget *button, gpointer user_data) {
  GtkWidget *radio, *entry;
  GtkWidget *dialog;
  gchar *filename, *nameonphone;
  gchar *errmsg;
  struct stat mystat;
  obex_t *handle;
  struct context global_context = {0,};
  obex_ctrans_t custfunc;
#ifdef HAVE_BLUETOOTH
  GtkWidget *channel_entry, *bdaddr_entry;
  gchar *bdstr;
  gchar *chan;
  bdaddr_t bdaddr;
  uint8_t channel;
#endif

  entry = gtk_object_get_data (GTK_OBJECT (user_data), "nameop");
  filename = gtk_editable_get_chars (GTK_EDITABLE (user_data), 0, -1);
  nameonphone = gtk_editable_get_chars (GTK_EDITABLE (entry), 0, -1);
  if ((strcmp (nameonphone, "") != 0) && (strstr (nameonphone, ".") != NULL)) {
    if (stat (filename, &mystat) == 0) {
      radio = gtk_object_get_data (GTK_OBJECT (user_data), "radio_irda");
      if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (radio)) == TRUE) {
	if ((handle = OBEX_Init (OBEX_TRANS_IRDA, obex_event, 0))) {
	  OBEX_SetUserData (handle, &global_context);
	  if (IrOBEX_TransportConnect (handle, "OBEX") >= 0) {
	    if (t68_connect (handle) == 0) {
	      dialog = messageDialog ("uploading file to your phone...\n\nlook at your phone\n", 0);
	      gtk_widget_show_all (dialog);
	      sendFile (handle, filename, nameonphone);
	      t68_disconnect (handle);
	      gtk_widget_destroy (dialog);
	    }
	    else
	      errorDialog ("can not connect to phone!");
	  }
	  else
	    errorDialog ("transport connect error!");
	}
      }
      else {
	radio = gtk_object_get_data (GTK_OBJECT (user_data), "radio_bt");
	if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (radio)) == TRUE) {
#ifdef HAVE_BLUETOOTH
	  if ((handle = OBEX_Init (OBEX_TRANS_BLUETOOTH, obex_event, 0))) {
	    bdaddr_entry = gtk_object_get_data (GTK_OBJECT (user_data), "bdaddr");
	    channel_entry = gtk_object_get_data (GTK_OBJECT (user_data),
						 "channel");
	    bdstr = gtk_editable_get_chars (GTK_EDITABLE (bdaddr_entry), 0, -1);
	    chan = gtk_editable_get_chars (GTK_EDITABLE (channel_entry), 0, -1);
	    OBEX_SetUserData (handle, &global_context);
	    str2ba (bdstr, &bdaddr);
	    channel = (uint8_t) atoi (chan);
	    if (BtOBEX_TransportConnect (handle, BDADDR_ANY, &bdaddr, channel) >= 0) {
	      if (t68_connect (handle) == 0) {
		dialog = messageDialog ("uploading file to your phone...\n\nlook at your phone\n", 0);
		gtk_widget_show_all (dialog);
		sendFile (handle, filename, nameonphone);
		t68_disconnect (handle);
		gtk_widget_destroy (dialog);
	      }
	      else
		errorDialog ("can not connect to phone!");
	    }
	    else
	      errorDialog ("Transport connect error!");
	  }
#else
	  errorDialog ("your version of tsemgr is\nNOT\ncompiled with bluetooth support!\n\nsee README for details");
#endif
	}
	else {
	  custfunc.customdata = cobex_open (config.device);
	  if ((handle = OBEX_Init (OBEX_TRANS_CUSTOM, obex_event, 0))) {
	    custfunc.connect = cobex_connect;
	    custfunc.disconnect = cobex_disconnect;
	    custfunc.write = cobex_write;
	    custfunc.handleinput = cobex_handle_input;
	    custfunc.listen = cobex_connect; /* Listen and connect is
						100% same on cable */
	    OBEX_RegisterCTransport (handle, &custfunc);
	    OBEX_SetUserData (handle, &global_context);
	    if (OBEX_TransportConnect (handle, (void*) 1, 0) >= 0) {
	      if (t68_connect (handle) == 0) {
		dialog = messageDialog ("uploading file to your phone...\n\nlook at your phone\n", 0);
		gtk_widget_show_all (dialog);
		sendFile (handle, filename, nameonphone);
		t68_disconnect (handle);
		gtk_widget_destroy (dialog);
	      }
	      else
		errorDialog ("can not connect to phone!");
	    }
	    else
	      errorDialog ("Transport connect error!");
	  }
	}
      }
    }
    else {
      errmsg = g_strdup_printf ("file not found:\n%s", filename);
      errorDialog (errmsg);
      g_free (errmsg);
    }
  }
  else
    errorDialog ("invalid name!\nname on phone field should contain\na name followed by an extension\n\nexample:\nmysound.emy");
}

void saveSMSSpace (GtkWidget *button, gpointer user_data) {
  gchar *message, *tmp, *curchar;
  const gchar *tostrip = " \n\t";
  gint point = 0;
  gint i;
  gint k = 0;
  gint j = 1;

  message = gtk_editable_get_chars (GTK_EDITABLE (user_data), 0, -1);
  tmp = g_malloc (sizeof (gchar) * strlen (message) + 1);
  gtk_editable_delete_text (GTK_EDITABLE (user_data), 0, -1);
  for (i = 0; i < strlen (message); i++, k++) {
    curchar = g_strdup_printf ("%c", message[i]);
    if ((j) && (strstr (tostrip, curchar) == NULL)) {
      tmp[k] = toupper (message[i]);
      j = 0;
    }
    else {
      tmp[k] = message[i];
      if (strstr (tostrip, curchar) != NULL) {
	j = 1;
	if (k > 0)
	  k--;
      }
    }
  }
  tmp[k] = '\0';
  g_strstrip (tmp);
  gtk_editable_insert_text (GTK_EDITABLE (user_data), tmp,
			    sizeof (gchar) * strlen (tmp), &point);
  g_free (message);
  g_free (curchar);
  g_free (tmp);
}

void setFilename (GtkWidget *button, gpointer user_data) {
  GtkWidget *entry;
  gchar *selected_filename;

  selected_filename = gtk_file_selection_get_filename (GTK_FILE_SELECTION (user_data));
  entry = gtk_object_get_data (GTK_OBJECT (user_data), "entry");
  gtk_entry_set_text (GTK_ENTRY (entry), selected_filename);
}

void createFileSelector (GtkWidget *button, gpointer user_data) {
  GtkWidget *file_selector;
  gchar *curpath;

  curpath = gtk_editable_get_chars (GTK_EDITABLE (user_data), 0, -1);
  file_selector = gtk_file_selection_new ("please select a file to upload");
  gtk_object_set_data (GTK_OBJECT (file_selector), "entry", user_data);
  gtk_signal_connect (GTK_OBJECT
		      (GTK_FILE_SELECTION(file_selector)->ok_button),
		      "clicked", GTK_SIGNAL_FUNC (setFilename),
		      file_selector);
  gtk_signal_connect_object (GTK_OBJECT
			     (GTK_FILE_SELECTION(file_selector)->ok_button),
			     "clicked", GTK_SIGNAL_FUNC (gtk_widget_destroy),
			     (gpointer) file_selector);
  gtk_signal_connect_object (GTK_OBJECT
			     (GTK_FILE_SELECTION(file_selector)->cancel_button),
			     "clicked", GTK_SIGNAL_FUNC (gtk_widget_destroy),
			     (gpointer) file_selector);
  gtk_file_selection_set_filename (GTK_FILE_SELECTION (file_selector), curpath);
  gtk_widget_show (file_selector);
}

void setNumber (GtkWidget *button, gpointer user_data) {
  GtkWidget *entry;
  gint row;
  gchar *number;
  GList *node = GTK_CLIST (user_data)->selection;

  if (node) {
    row = GPOINTER_TO_UINT (node->data);
    gtk_clist_get_text (GTK_CLIST (user_data), row, 2, &number);
    entry = gtk_object_get_data (GTK_OBJECT (user_data), "entry");
    gtk_entry_set_text (GTK_ENTRY (entry), number);
  }
}

void displayPhonebook (GtkWidget *button, gpointer user_data) {
  GtkWidget *dialog, *clist, *pbclist;
  GtkWidget *ok_button, *cancel_button;
  GtkWidget *scrolled_window;
  guint8 spacing;
  GdkPixmap *pix;
  GdkBitmap *mask;
  gchar *titles[3] = {"index", "name", "number"};
  gchar *entry[3];
  gint i;

  dialog = gtk_dialog_new ();
  gtk_widget_set_usize (dialog, 600, 300);
  ok_button = gtk_button_new_with_label ("ok");
  cancel_button = gtk_button_new_with_label ("cancel");
  clist = gtk_clist_new_with_titles (3, titles);
  scrolled_window = gtk_scrolled_window_new (NULL, NULL);
  gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolled_window),
				  GTK_POLICY_AUTOMATIC, GTK_POLICY_ALWAYS);
  gtk_clist_set_column_min_width (GTK_CLIST (clist), 1, 200);
  gtk_container_add (GTK_CONTAINER (scrolled_window), clist);
  gtk_object_set_data (GTK_OBJECT (clist), "entry", user_data);
  gtk_container_add (GTK_CONTAINER (GTK_DIALOG(dialog)->action_area),
		     ok_button);
  gtk_container_add (GTK_CONTAINER (GTK_DIALOG(dialog)->action_area),
		     cancel_button);
  gtk_container_add (GTK_CONTAINER (GTK_DIALOG(dialog)->vbox),
		     scrolled_window);
  gtk_widget_show_all (dialog);
  gtk_signal_connect (GTK_OBJECT (ok_button), "clicked",
		      GTK_SIGNAL_FUNC (setNumber), (gpointer) clist);
  gtk_signal_connect_object (GTK_OBJECT (ok_button), "clicked",
			     GTK_SIGNAL_FUNC (gtk_widget_destroy),
			     (gpointer) dialog);
  gtk_signal_connect_object (GTK_OBJECT (cancel_button), "clicked",
			     GTK_SIGNAL_FUNC (gtk_widget_destroy),
			     (gpointer) dialog);

  pbclist = gtk_object_get_data (GTK_OBJECT (mainwindow), "pbclist");
  if (GTK_CLIST (pbclist)->rows == 0)
    load_phonebook (NULL, (gpointer) pbclist);
  for (i = 0; i < GTK_CLIST (pbclist)->rows; i++) {
    gtk_clist_get_text (GTK_CLIST (pbclist), i, 0, &entry[0]);
    gtk_clist_get_pixtext (GTK_CLIST (pbclist), i, 1, &entry[1],
			   &spacing, &pix, &mask);
    gtk_clist_get_text (GTK_CLIST (pbclist), i, 2, &entry[2]);
    gtk_clist_append (GTK_CLIST (clist), entry);
    gtk_clist_set_pixtext (GTK_CLIST (clist), i, 1, entry[1], spacing, pix, mask);
  }
  gtk_signal_connect (GTK_OBJECT (clist), "click-column",
		      GTK_SIGNAL_FUNC (pbsortfunc), NULL);
}

void delete_selected_sms_ask (GtkWidget *button, gpointer user_data) {
  GtkWidget *dialog, *label;
  GtkWidget *okay_button, *cancel_button;
   
  if  ((gint) gtk_object_get_data (GTK_OBJECT (user_data),
				   "total_checkbuttons") != 0) {
    dialog = gtk_dialog_new ();
    label = gtk_label_new ("are you sure you want to\ndelete\n the selected messages\nfrom your phone?");
    okay_button = gtk_button_new_with_label ("Yes");
    cancel_button = gtk_button_new_with_label ("No");
   
    /* Ensure that the dialog box is destroyed when the user clicks ok. */
   
    gtk_signal_connect (GTK_OBJECT (okay_button), "clicked",
			GTK_SIGNAL_FUNC (delete_selected_sms), user_data);
    gtk_signal_connect_object (GTK_OBJECT (okay_button), "clicked",
			       GTK_SIGNAL_FUNC (gtk_widget_destroy),
			       (gpointer) dialog);

    gtk_signal_connect_object (GTK_OBJECT (cancel_button), "clicked",
			       GTK_SIGNAL_FUNC (gtk_widget_destroy),
			       (gpointer) dialog);
    gtk_container_add (GTK_CONTAINER (GTK_DIALOG(dialog)->action_area),
		       okay_button);
    gtk_container_add (GTK_CONTAINER (GTK_DIALOG(dialog)->action_area),
		       cancel_button);

    /* Add the label, and show everything we've added to the dialog. */

    gtk_container_add (GTK_CONTAINER (GTK_DIALOG(dialog)->vbox),
		       label);
    gtk_window_set_modal (GTK_WINDOW (dialog), TRUE);
    gtk_widget_show_all (dialog);
  }
  else
    errorDialog ("please load some\nmessages first!");
}

void delete_selected_sms (GtkWidget *button, gpointer user_data) {
  GtkWidget *checkbutton;
  gint i, total;
  gint totaldel = 0;
  gint index, mem;
  gchar *key;

  total = (gint) gtk_object_get_data (GTK_OBJECT (user_data), "total_checkbuttons");
  if (total != 0) {
    total /= 2;
    for (i = 0; i < total; i++) {
      key = g_strdup_printf ("checkbutton%d", i);
      checkbutton = (GtkWidget *) gtk_object_get_data (GTK_OBJECT (user_data), key);
      mem = (gint) gtk_object_get_data (GTK_OBJECT (user_data), "memtype");
      if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (checkbutton)) == TRUE) {
	key = g_strdup_printf ("index%d", i);
	index = (gint) gtk_object_get_data (GTK_OBJECT (user_data), key);
	if (deleteSMS (index, mem)) {
	  totaldel++;
	  sleep (2);
	}
      }
      g_free (key);
    }
  }
  else
    errorDialog ("please load some\nmessages first!");
  if (totaldel > 0)
    switch (mem) {
    case SIM:
      displaySMSListSIM (NULL, user_data);
      break;
    case ME:
      displaySMSListME (NULL, user_data);
      break;
    }
  else
    errorDialog ("please select the messages you\nwant to delete first");
}

void editPBentry (GtkWidget *button, gpointer user_data) {
  GtkWidget *entry, *radio;
  gchar *number, *name;
  gint index;

  entry = gtk_object_get_data (GTK_OBJECT (user_data), "name");
  name = gtk_entry_get_text (GTK_ENTRY (entry));
  entry = gtk_object_get_data (GTK_OBJECT (user_data), "number");
  number = gtk_entry_get_text (GTK_ENTRY (entry));
  index = (gint) gtk_object_get_data (GTK_OBJECT (user_data), "index");

  radio = gtk_object_get_data (GTK_OBJECT (user_data), "radio_home");
  if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (radio)) == TRUE)
    name = g_strdup_printf ("%s/H", name);
  else {
    radio = gtk_object_get_data (GTK_OBJECT (user_data), "radio_mobile");
    if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (radio)) == TRUE)
      name = g_strdup_printf ("%s/M", name);
    else {
      radio = gtk_object_get_data (GTK_OBJECT (user_data), "radio_work");
      if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (radio)) == TRUE)
	name = g_strdup_printf ("%s/W", name);
      else {
	radio = gtk_object_get_data (GTK_OBJECT (user_data), "radio_other");
	if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (radio)) == TRUE)
	  name = g_strdup_printf ("%s/O", name);
      }
    }
  }
  name = convert_to_gsm (name);
  writePBentry (index, number, name);
  sleep (1);
  load_phonebook (NULL, user_data);
  /*
  g_free (number);
  */
  g_free (name);
}

void editPBentryDialog (GtkWidget *button, gpointer user_data) {
  GtkWidget *dialog, *label, *entry;
  GtkWidget *ok_button, *cancel_button;
  GtkWidget *radio_home, *radio;
  GtkWidget *pixmap, *table;
  gchar *name, *number, *strindex;
  gchar *type, *tmp;
  gint index;
  guint8 spacing;
  GdkPixmap *pix;
  GdkBitmap *mask;
  guint row;
  GList *node = GTK_CLIST (user_data)->selection;

  if (node) {
    row = GPOINTER_TO_UINT (node->data);
    gtk_clist_get_text (GTK_CLIST (user_data), row, 0, &strindex);
    index = atoi (strindex);
    gtk_object_set_data (GTK_OBJECT (user_data), "index", GINT_TO_POINTER (index));
    gtk_clist_get_pixtext (GTK_CLIST (user_data), row, 1, &name, &spacing, &pix, &mask);
    gtk_clist_get_text (GTK_CLIST (user_data), row, 2, &number);
    tmp = g_strdup_printf ("type%d", row);
    type = gtk_object_get_data (GTK_OBJECT (user_data), tmp);
    
    dialog = gtk_dialog_new ();
    ok_button = gtk_button_new_with_label ("save");
    cancel_button = gtk_button_new_with_label ("cancel");
    
    table = gtk_table_new (4, 3, FALSE);
    gtk_container_add (GTK_CONTAINER (GTK_DIALOG(dialog)->vbox),
		       table);
    label = gtk_label_new ("name:");
    entry = gtk_entry_new_with_max_length (phoneproperties.pbnamelen);
    gtk_entry_set_text (GTK_ENTRY (entry), name);
    gtk_table_attach (GTK_TABLE (table), label, 0, 1, 0, 1,
		      GTK_SHRINK, GTK_SHRINK, 2, 2);
    gtk_table_attach (GTK_TABLE (table), entry, 1, 4, 0, 1,
		      GTK_SHRINK, GTK_SHRINK, 2, 2);
    gtk_object_set_data (GTK_OBJECT (user_data), "name", entry);

    label = gtk_label_new ("number:");
    entry = gtk_entry_new_with_max_length (phoneproperties.pbnumberlen);
    gtk_entry_set_text (GTK_ENTRY (entry), number);
    gtk_table_attach (GTK_TABLE (table), label, 0, 1, 1, 2,
		      GTK_SHRINK, GTK_SHRINK, 2, 2);
    gtk_table_attach (GTK_TABLE (table), entry, 1, 4, 1, 2,
		      GTK_SHRINK, GTK_SHRINK, 2, 2);
    gtk_object_set_data (GTK_OBJECT (user_data), "number", entry);

    radio_home = gtk_radio_button_new (NULL);
    pixmap = createPixmap (xpm_home);
    gtk_container_add (GTK_CONTAINER (radio_home), pixmap);
    gtk_table_attach (GTK_TABLE (table), radio_home, 0, 1, 2, 3,
		      GTK_SHRINK, GTK_SHRINK, 2, 2);
    gtk_widget_show (radio_home);
    gtk_object_set_data (GTK_OBJECT (user_data), "radio_home", radio_home);
    if (type[0] == 'H')
      gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (radio_home), TRUE);
    
    radio = gtk_radio_button_new_from_widget (GTK_RADIO_BUTTON (radio_home));
    pixmap = createPixmap (xpm_mobile);
    gtk_container_add (GTK_CONTAINER (radio), pixmap);
    gtk_table_attach (GTK_TABLE (table), radio, 1, 2, 2, 3,
		      GTK_SHRINK, GTK_SHRINK, 2, 2);
    gtk_widget_show (radio);
    gtk_object_set_data (GTK_OBJECT (user_data), "radio_mobile", radio);
    if (type[0] == 'M')
      gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (radio), TRUE);

    radio = gtk_radio_button_new_from_widget (GTK_RADIO_BUTTON (radio_home));
    pixmap = createPixmap (xpm_work);
    gtk_container_add (GTK_CONTAINER (radio), pixmap);
    gtk_table_attach (GTK_TABLE (table), radio, 2, 3, 2, 3,
		      GTK_SHRINK, GTK_SHRINK, 2, 2);
    gtk_widget_show (radio);
    gtk_object_set_data (GTK_OBJECT (user_data), "radio_work", radio);
    if (type[0] == 'W')
      gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (radio), TRUE);

    radio = gtk_radio_button_new_from_widget (GTK_RADIO_BUTTON (radio_home));
    pixmap = createPixmap (xpm_other);
    gtk_container_add (GTK_CONTAINER (radio), pixmap);
    gtk_table_attach (GTK_TABLE (table), radio, 3, 4, 2, 3,
		      GTK_SHRINK, GTK_SHRINK, 2, 2);
    gtk_widget_show (radio);
    gtk_object_set_data (GTK_OBJECT (user_data), "radio_other", radio);
    if (type[0] == 'O')
      gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (radio), TRUE);

    gtk_signal_connect (GTK_OBJECT (ok_button), "clicked",
			GTK_SIGNAL_FUNC (editPBentry), user_data);
    gtk_signal_connect_object (GTK_OBJECT (ok_button), "clicked",
			       GTK_SIGNAL_FUNC (gtk_widget_destroy),
			       (gpointer) dialog);

    gtk_signal_connect_object (GTK_OBJECT (cancel_button), "clicked",
			       GTK_SIGNAL_FUNC (gtk_widget_destroy),
			       (gpointer) dialog);

    gtk_container_add (GTK_CONTAINER (GTK_DIALOG(dialog)->action_area),
		       ok_button);
    gtk_container_add (GTK_CONTAINER (GTK_DIALOG(dialog)->action_area),
		       cancel_button);

    /* Add the label, and show everything we've added to the dialog. */
    
    gtk_window_set_modal (GTK_WINDOW (dialog), TRUE);

    gtk_widget_show_all (dialog);
  }
  else
    errorDialog ("please select\nan entry first!!");
}

void addPBentry (GtkWidget *button, gpointer user_data) {
  GtkWidget *entry, *radio;
  gchar *number, *name;

  entry = gtk_object_get_data (GTK_OBJECT (user_data), "name");
  name = gtk_entry_get_text (GTK_ENTRY (entry));
  entry = gtk_object_get_data (GTK_OBJECT (user_data), "number");
  number = gtk_entry_get_text (GTK_ENTRY (entry));

  radio = gtk_object_get_data (GTK_OBJECT (user_data), "radio_home");
  if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (radio)) == TRUE)
    name = g_strdup_printf ("%s/H", name);
  else {
    radio = gtk_object_get_data (GTK_OBJECT (user_data), "radio_mobile");
    if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (radio)) == TRUE)
      name = g_strdup_printf ("%s/M", name);
    else {
      radio = gtk_object_get_data (GTK_OBJECT (user_data), "radio_work");
      if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (radio)) == TRUE)
	name = g_strdup_printf ("%s/W", name);
      else {
	radio = gtk_object_get_data (GTK_OBJECT (user_data), "radio_other");
	if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (radio)) == TRUE)
	  name = g_strdup_printf ("%s/O", name);
      }
    }
  }
  name = convert_to_gsm (name);
  writePBentry (0, number, name);
  sleep (1);
  load_phonebook (NULL, user_data);
}

void addPBentryDialog (GtkWidget *button, gpointer user_data) {
  GtkWidget *dialog, *label, *entry;
  GtkWidget *ok_button, *cancel_button;
  GtkWidget *radio_home, *radio;
  GtkWidget *pixmap, *table;

  dialog = gtk_dialog_new ();
  ok_button = gtk_button_new_with_label ("add to phone");
  cancel_button = gtk_button_new_with_label ("cancel");

  table = gtk_table_new (4, 3, FALSE);
  gtk_container_add (GTK_CONTAINER (GTK_DIALOG(dialog)->vbox),
		     table);
  label = gtk_label_new ("name:");
  entry = gtk_entry_new_with_max_length (phoneproperties.pbnamelen);
  gtk_table_attach (GTK_TABLE (table), label, 0, 1, 0, 1,
		    GTK_SHRINK, GTK_SHRINK, 2, 2);
  gtk_table_attach (GTK_TABLE (table), entry, 1, 4, 0, 1,
		    GTK_SHRINK, GTK_SHRINK, 2, 2);
  gtk_object_set_data (GTK_OBJECT (user_data), "name", entry);

  label = gtk_label_new ("number:");
  entry = gtk_entry_new_with_max_length (phoneproperties.pbnumberlen);
  gtk_table_attach (GTK_TABLE (table), label, 0, 1, 1, 2,
		    GTK_SHRINK, GTK_SHRINK, 2, 2);
  gtk_table_attach (GTK_TABLE (table), entry, 1, 4, 1, 2,
		    GTK_SHRINK, GTK_SHRINK, 2, 2);
  gtk_object_set_data (GTK_OBJECT (user_data), "number", entry);

  radio_home = gtk_radio_button_new (NULL);
  pixmap = createPixmap (xpm_home);
  gtk_container_add (GTK_CONTAINER (radio_home), pixmap);
  gtk_table_attach (GTK_TABLE (table), radio_home, 0, 1, 2, 3,
		    GTK_SHRINK, GTK_SHRINK, 2, 2);
  gtk_widget_show (radio_home);
  gtk_object_set_data (GTK_OBJECT (user_data), "radio_home", radio_home);

  radio = gtk_radio_button_new_from_widget (GTK_RADIO_BUTTON (radio_home));
  pixmap = createPixmap (xpm_mobile);
  gtk_container_add (GTK_CONTAINER (radio), pixmap);
  gtk_table_attach (GTK_TABLE (table), radio, 1, 2, 2, 3,
		    GTK_SHRINK, GTK_SHRINK, 2, 2);
  gtk_widget_show (radio);
  gtk_object_set_data (GTK_OBJECT (user_data), "radio_mobile", radio);

  radio = gtk_radio_button_new_from_widget (GTK_RADIO_BUTTON (radio_home));
  pixmap = createPixmap (xpm_work);
  gtk_container_add (GTK_CONTAINER (radio), pixmap);
  gtk_table_attach (GTK_TABLE (table), radio, 2, 3, 2, 3,
		    GTK_SHRINK, GTK_SHRINK, 2, 2);
  gtk_widget_show (radio);
  gtk_object_set_data (GTK_OBJECT (user_data), "radio_work", radio);

  radio = gtk_radio_button_new_from_widget (GTK_RADIO_BUTTON (radio_home));
  pixmap = createPixmap (xpm_other);
  gtk_container_add (GTK_CONTAINER (radio), pixmap);
  gtk_table_attach (GTK_TABLE (table), radio, 3, 4, 2, 3,
		    GTK_SHRINK, GTK_SHRINK, 2, 2);
  gtk_widget_show (radio);
  gtk_object_set_data (GTK_OBJECT (user_data), "radio_other", radio);

  gtk_signal_connect (GTK_OBJECT (ok_button), "clicked",
		      GTK_SIGNAL_FUNC (addPBentry), user_data);
  gtk_signal_connect_object (GTK_OBJECT (ok_button), "clicked",
			     GTK_SIGNAL_FUNC (gtk_widget_destroy),
			     (gpointer) dialog);

  gtk_signal_connect_object (GTK_OBJECT (cancel_button), "clicked",
			     GTK_SIGNAL_FUNC (gtk_widget_destroy),
			     (gpointer) dialog);

  gtk_container_add (GTK_CONTAINER (GTK_DIALOG(dialog)->action_area),
		     ok_button);
  gtk_container_add (GTK_CONTAINER (GTK_DIALOG(dialog)->action_area),
		     cancel_button);

  /* Add the label, and show everything we've added to the dialog. */

  gtk_window_set_modal (GTK_WINDOW (dialog), TRUE);

  gtk_widget_show_all (dialog);
}

void delete_phonebook_entries (GtkWidget *button, gpointer user_data) {
  gchar *strindex;
  gint index;
  guint row;
  GList *node = GTK_CLIST (user_data)->selection;

  while (node) {
    row = GPOINTER_TO_UINT (node->data);
    gtk_clist_get_text (GTK_CLIST (user_data), row, 0, &strindex);
    index = atoi (strindex);
    sleep (1);
    writePBentry (index, "", "");
    gtk_clist_remove (GTK_CLIST (user_data), row);
    node = GTK_CLIST (user_data)->selection;
  }
}

void load_phonebook (GtkWidget *button, gpointer user_data) {
  GtkWidget *dialog, *progress_bar;
  GtkWidget *label;
  GtkStyle *style;
  GdkPixmap *pixmap;
  GdkBitmap *mask;
  v24_port_t *port = NULL;
  gint stop = 0;
  gfloat i = 1.0;
  gchar *tmp;
  gchar **entry;
  gchar *entrys[3];
  gchar *blar[3];
  gchar *command;
  gchar buffer[512];
  gchar *formatstring;
  gchar buffer1[512];
  char *b1_ptr, *b2_ptr;
  iconv_t cd; 
  size_t size, left;

  gtk_clist_clear (GTK_CLIST (user_data));
  dialog = PBstatusDialog ();
  progress_bar = gtk_object_get_data (GTK_OBJECT (dialog), "progress_bar");
  label = gtk_object_get_data (GTK_OBJECT (dialog), "label");
  gtk_progress_set_activity_mode (GTK_PROGRESS (progress_bar), TRUE);
  gtk_widget_show_all (dialog);
  port = v24OpenPort (config.device, V24_RTS_CTS);
  if (port != NULL) {
    selectPBstorage (port, "ME");

    command = g_strdup_printf ("AT+CSCS=\"UTF-8\"\r");
    v24Puts (port, command);
    g_free (command);
    v24Gets (port, buffer, 512);
    v24Gets (port, buffer, 512);
    
    command = g_strdup_printf ("AT+CPBR=1,250\r");
    v24Puts (port, command);
    g_free (command);
    v24Gets (port, buffer, 512);
    
    while (!stop) {
      v24Gets (port, buffer, 512);
      cd = iconv_open ("ISO8859-2", "UTF8");
      size = sizeof (buffer);
      left = sizeof (buffer1);
      b2_ptr = buffer1;
      b1_ptr = buffer;
      iconv (cd, &b1_ptr, &size, &b2_ptr, &left);
      iconv_close (cd);
      //strncpy (buffer, buffer1, strlen(buffer));
      g_strstrip (buffer1);

      gtk_progress_set_value ((GtkProgress *) progress_bar, i);
      formatstring = g_strdup_printf ("entry #%d", (gint) i);
      gtk_label_set_text (GTK_LABEL (label), formatstring);
      
      while (gtk_events_pending())
	gtk_main_iteration();
      
      if (strstr (buffer1, "+CPBR:")) {
	entry = g_strsplit (buffer1 + 6, ",", 3);
	entrys[0] = g_strdup (entry[0]);
	entrys[1] = g_strndup (entry[3] + 1, strlen (entry[3] + 1) - 3);
	
	entrys[1] = convert_from_gsm (entrys[1]);
	style = gtk_widget_get_style (mainwindow);
	tmp = g_strdup_printf ("type%d", (gint) i - 1);
	
	switch (entry[3][strlen (entry[3] + 1) - 1]) {
	  case 'H' :
	    pixmap = gdk_pixmap_create_from_xpm_d (mainwindow->window,  &mask,
					           &style->bg[GTK_STATE_NORMAL],
						   (gchar **) xpm_home);
  	    gtk_object_set_data (GTK_OBJECT (user_data), tmp, "H");
  	    break;
  	  case 'M' :
  	    pixmap = gdk_pixmap_create_from_xpm_d (mainwindow->window,  &mask,
 						   &style->bg[GTK_STATE_NORMAL],
						   (gchar **) xpm_mobile);
	    gtk_object_set_data (GTK_OBJECT (user_data), tmp, "M");
	    break;
	  case 'O' :
	    pixmap = gdk_pixmap_create_from_xpm_d (mainwindow->window,  &mask,
						   &style->bg[GTK_STATE_NORMAL],
						   (gchar **) xpm_other);
	    gtk_object_set_data (GTK_OBJECT (user_data), tmp, "O");
	    break;
	  case 'W' :
	    pixmap = gdk_pixmap_create_from_xpm_d (mainwindow->window,  &mask,
						   &style->bg[GTK_STATE_NORMAL],
						   (gchar **) xpm_work);
	    gtk_object_set_data (GTK_OBJECT (user_data), tmp, "W");
	    break;
	  case 'F' :
	    pixmap = gdk_pixmap_create_from_xpm_d (mainwindow->window,  &mask,
						   &style->bg[GTK_STATE_NORMAL],
						   (gchar **) xpm_fax);
	    gtk_object_set_data (GTK_OBJECT (user_data), tmp, "F");
	    break;
	  default :
	    g_print ("%c\n", entry[3][strlen (entry[3] + 1) - 1]);
	    break;
	}
	
	entrys[2] = g_strndup (entry[1] + 1, strlen (entry[1] + 1) - 1);
	
	if ((entrys[2][0] != '+') && (strstr (entry[2], "145")))
	  entrys[2] = g_strdup_printf ("+%s", entrys[2]);
	
	gtk_clist_append (GTK_CLIST (user_data), entrys);
	//      g_print ("%s\n", entrys[1]);
	gtk_clist_set_pixtext (GTK_CLIST (user_data), (gint) i - 1, 1,
			       entrys[1], 2, pixmap, mask);
      }
      if (strcasecmp (buffer, "OK\r\n") == 0)
	stop = 1;
      if (strcasecmp (buffer, "ERROR\r\n") == 0)
	stop = 1;
      
      i++;
    }
    
    v24ClosePort (port);
  }
  else
    errorDialog ("unable to open device!\nplease check if your phone is connected\nand if you specified the correct device!");
  
  gtk_widget_destroy (dialog);
}

void save_selected (GtkWidget *button, gpointer user_data) {
  GtkWidget *subtable, *checkbutton;
  GtkWidget *label;
  GtkWidget *file_selector;
  gchar *tmp;
  gchar *to_save;
  gchar *key;
  gint i, total;
  gint selected = 0;

  to_save = g_strdup ("");
  total = (gint) gtk_object_get_data (GTK_OBJECT (user_data), "total_checkbuttons");
  if (total != 0) {
    total /= 2;

    for (i = 0; i < total; i++) {
      key = g_strdup_printf ("subtable%d", i);
      subtable = gtk_object_get_data (GTK_OBJECT (user_data), key);

      key = g_strdup_printf ("checkbutton%d", i);
      checkbutton = gtk_object_get_data (GTK_OBJECT (user_data), key);

      if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (checkbutton)) == TRUE) {
	label = gtk_object_get_data (GTK_OBJECT (subtable), "number");
	gtk_label_get (GTK_LABEL (label), &tmp);
	to_save = g_strdup_printf ("%s%s", to_save, tmp);

	label = gtk_object_get_data (GTK_OBJECT (subtable), "date");
	gtk_label_get (GTK_LABEL (label), &tmp);
	to_save = g_strdup_printf ("%s%s", to_save, tmp);

	label = gtk_object_get_data (GTK_OBJECT (subtable), "message");
	gtk_label_get (GTK_LABEL (label), &tmp);
	to_save = g_strdup_printf ("%s%s\n----------------------------------------------",
				   to_save, tmp);
	selected++;
      }
    }
    if (selected != 0) {
      to_save = g_strdup_printf ("%s\n", to_save);
      file_selector = gtk_file_selection_new("please select a file to save to");
      gtk_object_set_data (GTK_OBJECT (file_selector), "to_save", to_save);
    
      gtk_signal_connect (GTK_OBJECT
			  (GTK_FILE_SELECTION(file_selector)->ok_button),
			  "clicked", GTK_SIGNAL_FUNC (store_filename),
			  (gpointer) file_selector);
   			   
      /* Ensure that the dialog box is destroyed when the user clicks a button. */
   
      gtk_signal_connect_object (GTK_OBJECT
				 (GTK_FILE_SELECTION(file_selector)->cancel_button),
				 "clicked", GTK_SIGNAL_FUNC (gtk_widget_destroy),
				 (gpointer) file_selector);
    
      /* Display that dialog */
   
      gtk_widget_show (file_selector);
    }
    else
      errorDialog ("please select one\nor more messages first!");
  }
  else
    errorDialog ("please load some\nmessages first!");
}

void select_all (GtkWidget *button, gpointer user_data) {
  GtkWidget *checkbutton;
  gint i, total;
  gchar *key;

  total = (gint) gtk_object_get_data (GTK_OBJECT (user_data), "total_checkbuttons");
  if (total != 0) {
    total /= 2;
    for (i = 0; i < total; i++) {
      key = g_strdup_printf ("checkbutton%d", i);
      checkbutton = (GtkWidget *) gtk_object_get_data (GTK_OBJECT (user_data), key);
      gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (checkbutton), TRUE);
      g_free (key);
    }
  }
  else
    errorDialog ("please load some\nmessages first!");
}

void updateStatus (GtkWidget *textfield, gpointer user_data) {
  gchar *label;
  gint len;
  //  gint num, curlen;

  len = gtk_text_get_length (GTK_TEXT (textfield));
  /*
  if (len <= 1280) {
    num = len / 160;
    if (!num)
      num = 1;
    if (len % 160) {
      if (len / 160 != 0)
	num++;
      curlen = len % 160;
    }
    else
      if ((num == 1) && (len != 160))
	curlen = 0;
      else
	curlen = 160;
    label = g_strdup_printf ("SMS #%d : %d/160", num, curlen);
    gtk_label_set_text (GTK_LABEL (user_data), label);
  }
  */
  if (len <= 160) {
    label = g_strdup_printf ("SMS #1 : %d/160", len);
    gtk_label_set_text (GTK_LABEL (user_data), label);
  }
  else
    gtk_text_backward_delete (GTK_TEXT (textfield), len - 160);
}

void sendSMS (GtkWidget *button, gpointer user_data) {
  GtkWidget *textfield, *checkbutton;
  gchar *message, *number;
  gint len, i;

  number = g_malloc (sizeof (gchar) * 200);

  number = gtk_editable_get_chars (GTK_EDITABLE (user_data), 0, -1);

  if (pNumber (number)) {
    textfield = (GtkWidget *) gtk_object_get_data (GTK_OBJECT (user_data), "message");
    checkbutton = (GtkWidget *) gtk_object_get_data (GTK_OBJECT (user_data), "store");
    len = gtk_text_get_length (GTK_TEXT (textfield));
    message = g_malloc (sizeof (gchar) * len + 1);

    for (i = 0; i < len; i++)
      message[i] = GTK_TEXT_INDEX (GTK_TEXT (textfield), i);
    message[i] = '\0';
    g_print ("%s\n%s\n", number, message);

    if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (checkbutton)) == FALSE) {
      if (encode_and_sendSMS (number, message, 0) == 0)
	errorDialog ("sending failed!");
      else
	messageDialog ("message succesfully sent!", 1);
    }
    else {
      if (encode_and_sendSMS (number, message, 1) == 0)
	errorDialog ("sending failed!");
      else
	messageDialog ("message sent succesfull!", 1);
    }
    g_free (message);
  }
  else
    errorDialog ("please enter a valid number\nexamples:\n+49170123456\n0170123456");
  g_free (number);
}

void store_filename (GtkFileSelection *selector, gpointer user_data) {
  gchar *selected_filename, *to_save;
  int fd;

  selected_filename = gtk_file_selection_get_filename (GTK_FILE_SELECTION(user_data));
  to_save = (gchar *) gtk_object_get_data (GTK_OBJECT (user_data), "to_save");

  fd = open (selected_filename, O_RDWR | O_CREAT | O_EXCL, 0666);
  if (fd != -1) {
    if (write (fd, to_save, sizeof (gchar) * strlen (to_save)) == -1)
      g_print ("error on writing to %s\n", selected_filename);
    gtk_widget_destroy (user_data);
  }
  else if (errno == EEXIST)
    save_over_ask (user_data);
  close (fd);
}

void save_over (GtkWidget *button, gpointer user_data) {
  gchar *selected_filename, *to_save;
  int fd;

  selected_filename = gtk_file_selection_get_filename (GTK_FILE_SELECTION
						       (user_data));
  to_save = (gchar *) gtk_object_get_data (GTK_OBJECT (user_data), "to_save");

  fd = open (selected_filename, O_RDWR | O_CREAT, 0666);
  if (fd != -1)
    write (fd, to_save, sizeof (gchar) * strlen (to_save));
  else
    g_print ("error on saving %s\nerrorno: %d\n", selected_filename, errno);
  close (fd);
}

void saveSMS (GtkWidget *widget, gpointer data) {
  GtkWidget *label;
  GtkWidget *file_selector;
  gchar *tmp;
  gchar *to_save;

  to_save = g_strdup ("");

  label = gtk_object_get_data (GTK_OBJECT(data), "number");
  gtk_label_get (GTK_LABEL (label), &tmp);
  to_save = g_strdup_printf ("%s%s", to_save, tmp);

  label = gtk_object_get_data (GTK_OBJECT(data), "date");
  gtk_label_get (GTK_LABEL (label), &tmp);
  to_save = g_strdup_printf ("%s%s", to_save, tmp);

  label = gtk_object_get_data (GTK_OBJECT(data), "message");
  gtk_label_get (GTK_LABEL (label), &tmp);
  to_save = g_strdup_printf ("%s%s", to_save, tmp);

  file_selector = gtk_file_selection_new("please select a file to save to");
  gtk_object_set_data (GTK_OBJECT (file_selector), "to_save", to_save);
   
  gtk_signal_connect (GTK_OBJECT
		      (GTK_FILE_SELECTION(file_selector)->ok_button),
		      "clicked", GTK_SIGNAL_FUNC (store_filename),
		      (gpointer) file_selector);
   			   
   /* Ensure that the dialog box is destroyed when the user clicks a button. */
   
  gtk_signal_connect_object (GTK_OBJECT
			     (GTK_FILE_SELECTION(file_selector)->cancel_button),
			     "clicked", GTK_SIGNAL_FUNC (gtk_widget_destroy),
			     (gpointer) file_selector);
   
  /* Display that dialog */
   
  gtk_widget_show (file_selector);

}

void destroy (GtkWidget *widget, gpointer data) {
  gtk_main_quit();
}
