/*
  tsemgr is a gtk application to manage a sonyericsson t68 mobile phone
  Copyright (C) 2003 Sebastian Sareyko

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/


#include "config.h"
#include <stdio.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <errno.h>
#include "interface.h"
#include "sms.h"
#include "callbacks.h"
#include "remote.h"
#include "pixmaps.h"
#include "misc.h"

static const gchar *abouttext =
"tsemgr is a gtk application to manage a SonyEricsson T68m/i mobile phone\n"
"Copyright (C) 2003 Sebastian Sareyko\n"
"\n"
"This program is free software; you can redistribute it and/or\n"
"modify it under the terms of the GNU General Public License\n"
"as published by the Free Software Foundation; either version 2\n"
"of the License, or (at your option) any later version.\n"
"\n"
"This program is distributed in the hope that it will be useful,\n"
"but WITHOUT ANY WARRANTY; without even the implied warranty of\n"
"MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the\n"
"GNU General Public License for more details.\n"
"\n"
"You should have received a copy of the GNU General Public License\n"
"along with this program; if not, write to the Free Software\n"
"Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.";

void delete_phonebook_entries_ask (GtkWidget *button, gpointer user_data) {
  GtkWidget *dialog, *label;
  GtkWidget *okay_button, *cancel_button;
  GList *node = GTK_CLIST (user_data)->selection;
  /* Create the widgets */
   
  if (node) {
    dialog = gtk_dialog_new ();
    label = gtk_label_new ("are you sure you want to\ndelete\n the selected entries\nfrom your phone?");
    okay_button = gtk_button_new_with_label ("Yes");
    cancel_button = gtk_button_new_with_label ("No");
   
    /* Ensure that the dialog box is destroyed when the user clicks ok. */
   
    gtk_signal_connect (GTK_OBJECT (okay_button), "clicked",
			GTK_SIGNAL_FUNC (delete_phonebook_entries), user_data);
    gtk_signal_connect_object (GTK_OBJECT (okay_button), "clicked",
			       GTK_SIGNAL_FUNC (gtk_widget_destroy),
			       (gpointer) dialog);

    gtk_signal_connect_object (GTK_OBJECT (cancel_button), "clicked",
			       GTK_SIGNAL_FUNC (gtk_widget_destroy),
			       (gpointer) dialog);
    gtk_container_add (GTK_CONTAINER (GTK_DIALOG(dialog)->action_area),
		       okay_button);
    gtk_container_add (GTK_CONTAINER (GTK_DIALOG(dialog)->action_area),
		       cancel_button);

    /* Add the label, and show everything we've added to the dialog. */

    gtk_container_add (GTK_CONTAINER (GTK_DIALOG(dialog)->vbox),
		       label);
    gtk_window_set_modal (GTK_WINDOW (dialog), TRUE);
    gtk_widget_show_all (dialog);
  }
  else
    errorDialog ("please select\nan entry first!!");
}

GtkWidget *createPixmap (gchar **which) {
  GtkWidget *pixmapwid;
  GtkStyle *style;
  GdkPixmap *pixmap;
  GdkBitmap *mask;

  style = gtk_widget_get_style (mainwindow);
  pixmap = gdk_pixmap_create_from_xpm_d (mainwindow->window,  &mask,
					 &style->bg[GTK_STATE_NORMAL],
					 which);

  pixmapwid = gtk_pixmap_new (pixmap, mask);

  return pixmapwid;
}

GtkWidget *messageDialog (gchar *msg, gint showbutton) {
  GtkWidget *dialog, *label, *ok_button;

  dialog = gtk_dialog_new ();
  label = gtk_label_new (msg);
  if (showbutton)
    ok_button = gtk_button_new_with_label ("Ok");
  gtk_container_add (GTK_CONTAINER (GTK_DIALOG(dialog)->vbox), label);
  if (showbutton)
    gtk_container_add (GTK_CONTAINER (GTK_DIALOG(dialog)->action_area), ok_button);
  gtk_widget_show (label);
  if (showbutton)
    gtk_widget_show (ok_button);
  gtk_widget_show (dialog);
  gtk_window_set_title (GTK_WINDOW (dialog), "message");
  gtk_window_set_modal (GTK_WINDOW (dialog), TRUE);
  if (showbutton)
    gtk_signal_connect_object (GTK_OBJECT (ok_button), "clicked",
			       GTK_SIGNAL_FUNC (gtk_widget_destroy),
			       GTK_OBJECT (dialog));
  return dialog;
}

void errorDialog (gchar *errmsg) {
  GtkWidget *dialog, *label, *table;
  GtkWidget *frame, *ok_button;
  GtkWidget *pixmap;

  dialog = gtk_dialog_new ();
  label = gtk_label_new (errmsg);
  ok_button = gtk_button_new_with_label ("Ok");
  frame = gtk_frame_new ((gchar *) NULL);
  table = gtk_table_new (2, 1, FALSE);
  pixmap = createPixmap (xpm_quit);
  gtk_table_attach_defaults (GTK_TABLE (table), pixmap, 0, 1, 0, 1);
  gtk_table_attach_defaults (GTK_TABLE (table), label, 1, 2, 0, 1);
  gtk_container_add (GTK_CONTAINER (frame), table);
  gtk_container_add (GTK_CONTAINER (GTK_DIALOG(dialog)->vbox), frame);
  gtk_container_add (GTK_CONTAINER (GTK_DIALOG(dialog)->action_area), ok_button);
  /*
  gtk_widget_show (frame);
  gtk_widget_show (label);
  gtk_widget_show (ok_button);
  */
  gtk_widget_show_all (dialog);
  gtk_window_set_title (GTK_WINDOW (dialog), "error");
  gtk_window_set_modal (GTK_WINDOW (dialog), TRUE);
  gtk_signal_connect_object (GTK_OBJECT (ok_button), "clicked",
			     GTK_SIGNAL_FUNC (gtk_widget_destroy),
			     GTK_OBJECT (dialog));
}

GtkWidget *PBstatusDialog () {
  GtkWidget *dialog, *label;
  GtkWidget *progress_bar;
   
  dialog = gtk_dialog_new ();
  label = gtk_label_new ("loading phonebook...");
  progress_bar = gtk_progress_bar_new ();
  gtk_progress_set_format_string ((GtkProgress *) progress_bar, "entry #%v");
  gtk_progress_set_show_text ((GtkProgress *) progress_bar, TRUE);
  gtk_object_set_data (GTK_OBJECT (dialog), "progress_bar", progress_bar);
  gtk_container_add (GTK_CONTAINER (GTK_DIALOG(dialog)->vbox), label);
  gtk_container_add (GTK_CONTAINER (GTK_DIALOG(dialog)->vbox), progress_bar);
  label = gtk_label_new ("entry #0");
  gtk_object_set_data (GTK_OBJECT (dialog), "label", label);
  gtk_container_add (GTK_CONTAINER (GTK_DIALOG(dialog)->vbox), label);
  gtk_window_set_modal (GTK_WINDOW (dialog), TRUE);
  return dialog;
}

GtkWidget *SMSstatusDialog () {
  GtkWidget *dialog, *label, *statuslabel;
  GtkWidget *progress_bar;
   
  dialog = gtk_dialog_new ();
  label = gtk_label_new ("loading messages...");
  statuslabel = gtk_label_new ("message #1");
  progress_bar = gtk_progress_bar_new ();
  gtk_progress_set_activity_mode (GTK_PROGRESS (progress_bar), TRUE);
  gtk_progress_set_format_string ((GtkProgress *) progress_bar, "message #%v");
  gtk_progress_set_show_text ((GtkProgress *) progress_bar, TRUE);
  gtk_object_set_data (GTK_OBJECT (dialog), "progress_bar", progress_bar);
  gtk_object_set_data (GTK_OBJECT (progress_bar), "statuslabel", statuslabel);
  gtk_container_add (GTK_CONTAINER (GTK_DIALOG(dialog)->vbox), label);
  gtk_container_add (GTK_CONTAINER (GTK_DIALOG(dialog)->vbox), progress_bar);
  gtk_container_add (GTK_CONTAINER (GTK_DIALOG(dialog)->vbox), statuslabel);
  gtk_window_set_modal (GTK_WINDOW (dialog), TRUE);
  return dialog;
}

void save_over_ask (gpointer user_data) {
  GtkWidget *dialog, *label;
  GtkWidget *okay_button, *cancel_button;
  gchar *message, *filename;
  /* Create the widgets */
   
  filename = gtk_file_selection_get_filename (GTK_FILE_SELECTION (user_data));
  dialog = gtk_dialog_new ();
  message = g_strdup_printf ("file %s already exists!\noverwrite?", filename);
  label = gtk_label_new (message);
  okay_button = gtk_button_new_with_label ("Ok");
  cancel_button = gtk_button_new_with_label ("Cancel");
   
  /* Ensure that the dialog box is destroyed when the user clicks ok. */
   
  gtk_signal_connect (GTK_OBJECT (okay_button), "clicked",
		      GTK_SIGNAL_FUNC (save_over), user_data);
  gtk_signal_connect_object (GTK_OBJECT (okay_button), "clicked",
			     GTK_SIGNAL_FUNC (gtk_widget_destroy),
			     (gpointer) user_data);
  gtk_signal_connect_object (GTK_OBJECT (okay_button), "clicked",
			     GTK_SIGNAL_FUNC (gtk_widget_destroy),
			     (gpointer) dialog);

  gtk_signal_connect_object (GTK_OBJECT (cancel_button), "clicked",
			     GTK_SIGNAL_FUNC (gtk_widget_destroy),
			     (gpointer) dialog);
  gtk_container_add (GTK_CONTAINER (GTK_DIALOG(dialog)->action_area),
		     okay_button);
  gtk_container_add (GTK_CONTAINER (GTK_DIALOG(dialog)->action_area),
		     cancel_button);

  /* Add the label, and show everything we've added to the dialog. */

  gtk_container_add (GTK_CONTAINER (GTK_DIALOG(dialog)->vbox),
		     label);
  gtk_widget_show_all (dialog);
}

GtkWidget *createSMSreadContent () {
  GtkWidget *table;
  GtkWidget *button;
  GtkWidget *box;
  GtkWidget *scrolled_window;

  table = gtk_table_new (2, 2, FALSE);
  gtk_widget_set_usize (table, 600, 300);

  box = gtk_hbutton_box_new ();
  gtk_widget_show (box);
  gtk_table_attach (GTK_TABLE (table), box, 0, 1, 0, 1,
		    GTK_SHRINK, GTK_SHRINK, 0, 0);

  button = gtk_button_new_with_label ("...from sim");

  scrolled_window = gtk_scrolled_window_new (NULL, NULL);
  gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolled_window),
				  GTK_POLICY_AUTOMATIC, GTK_POLICY_ALWAYS);
  gtk_table_attach_defaults (GTK_TABLE (table), scrolled_window,
			     0, 2, 1, 2);

  gtk_signal_connect (GTK_OBJECT (button), "clicked",
		      GTK_SIGNAL_FUNC (displaySMSListSIM),
		      (gpointer) scrolled_window);
  gtk_container_add (GTK_CONTAINER (box), button);
  gtk_button_box_set_layout (GTK_BUTTON_BOX (box), GTK_BUTTONBOX_START);
  gtk_button_box_set_child_size (GTK_BUTTON_BOX (box), 85, 20);
  gtk_widget_show (button);

  button = gtk_button_new_with_label ("...from phone");
  gtk_signal_connect (GTK_OBJECT (button), "clicked",
		      GTK_SIGNAL_FUNC (displaySMSListME),
		      (gpointer) scrolled_window);
  gtk_container_add (GTK_CONTAINER (box), button);
  gtk_object_set_data (GTK_OBJECT (scrolled_window), "table", 0);
  gtk_widget_show (button);

  box = gtk_hbutton_box_new ();
  gtk_widget_show (box);
  gtk_table_attach (GTK_TABLE (table), box, 1, 2, 0, 1,
		    GTK_SHRINK, GTK_SHRINK, 0, 0);

  button = gtk_button_new_with_label ("select all");
  gtk_signal_connect (GTK_OBJECT (button), "clicked",
		      GTK_SIGNAL_FUNC (select_all),
		      (gpointer) scrolled_window);
  gtk_container_add (GTK_CONTAINER (box), button);
  gtk_widget_show (button);

  button = gtk_button_new_with_label ("save selected");
  gtk_signal_connect (GTK_OBJECT (button), "clicked",
		      GTK_SIGNAL_FUNC (save_selected),
		      (gpointer) scrolled_window);
  gtk_container_add (GTK_CONTAINER (box), button);
  gtk_widget_show (button);

  button = gtk_button_new_with_label ("delete selected");
  gtk_signal_connect (GTK_OBJECT (button), "clicked",
		      GTK_SIGNAL_FUNC (delete_selected_sms_ask),
		      (gpointer) scrolled_window);
  gtk_container_add (GTK_CONTAINER (box), button);
  gtk_widget_show (button);

  gtk_widget_show (scrolled_window);
  return table;
}

GtkWidget *createSMSsendContent (void) {
  GtkWidget *table, *entry, *label, *status;
  GtkWidget *textfield, *button, *checkbutton;
  GtkWidget *pbbutton, *magicbutton;
  GtkWidget *pixmapwid;

  table = gtk_table_new (4, 3, FALSE);
  label = gtk_label_new ("...to:");
  status = gtk_label_new ("SMS #1 : 0/160");
  entry = gtk_entry_new_with_max_length (22);
  checkbutton = gtk_check_button_new_with_label ("store on phone only (outbox)");
  textfield = gtk_text_new (NULL, NULL);
  gtk_widget_set_usize (textfield, 400, 200);
  button = gtk_button_new_with_label ("send");
  pbbutton = gtk_button_new ();
  magicbutton = gtk_button_new_with_label ("reduce length");
  pixmapwid = createPixmap (xpm_phonebook);
  gtk_container_add (GTK_CONTAINER (pbbutton), pixmapwid);
  gtk_text_set_editable (GTK_TEXT (textfield), TRUE);
  gtk_table_attach (GTK_TABLE (table), label, 0, 1, 0, 1,
		    GTK_EXPAND, GTK_SHRINK, 10, 10);
  gtk_table_attach (GTK_TABLE (table), entry, 1, 2, 0, 1,
		    GTK_EXPAND, GTK_SHRINK, 10, 10);
  gtk_table_attach (GTK_TABLE (table), pbbutton, 2, 3, 0, 1,
		    GTK_EXPAND, GTK_SHRINK, 10, 10);
  gtk_table_attach (GTK_TABLE (table), checkbutton, 3, 4, 0, 1,
		    GTK_EXPAND, GTK_SHRINK, 10, 10);
  gtk_table_attach (GTK_TABLE (table), textfield, 0, 4, 1, 2,
		    GTK_EXPAND, GTK_EXPAND | GTK_FILL, 0, 0);
  gtk_table_attach (GTK_TABLE (table), status, 0, 2, 2, 3,
		    GTK_EXPAND, GTK_SHRINK, 10, 10);
  gtk_table_attach (GTK_TABLE (table), magicbutton, 2, 3, 2, 3,
		    GTK_EXPAND, GTK_SHRINK, 10, 10);
  gtk_table_attach (GTK_TABLE (table), button, 3, 4, 2, 3,
		    GTK_EXPAND, GTK_SHRINK, 10, 10);
  gtk_widget_show (label);
  gtk_widget_show (entry);
  gtk_widget_show (pbbutton);
  gtk_widget_show (pixmapwid);
  gtk_widget_show (checkbutton);
  gtk_widget_show (textfield);
  gtk_widget_show (status);
  gtk_widget_show (magicbutton);
  gtk_widget_show (button);

  gtk_object_set_data (GTK_OBJECT (entry), "message", textfield);
  gtk_object_set_data (GTK_OBJECT (entry), "store", checkbutton);
  gtk_signal_connect (GTK_OBJECT (pbbutton), "clicked",
		      GTK_SIGNAL_FUNC (displayPhonebook),
		      (gpointer) entry);
  gtk_signal_connect (GTK_OBJECT (button), "clicked",
		      GTK_SIGNAL_FUNC (sendSMS),
		      (gpointer) entry);
  gtk_signal_connect (GTK_OBJECT (magicbutton), "clicked",
		      GTK_SIGNAL_FUNC (saveSMSSpace),
		      (gpointer) textfield);
  gtk_signal_connect (GTK_OBJECT (textfield), "changed",
		      GTK_SIGNAL_FUNC (updateStatus),
		      (gpointer) status);

  return table;
}

GtkWidget *create_sms_sub_tab (void) {
  GtkWidget *label;
  GtkWidget *mesg;
  GtkWidget *subnotebook;

  subnotebook = gtk_notebook_new ();
  gtk_notebook_set_tab_pos (GTK_NOTEBOOK (subnotebook), GTK_POS_LEFT);
  gtk_widget_show (subnotebook);

  label = gtk_label_new ("read...");
  mesg = createSMSreadContent ();
  gtk_widget_show (mesg);
  gtk_notebook_append_page (GTK_NOTEBOOK (subnotebook), mesg, label);
  gtk_widget_show (label);

  label = gtk_label_new ("send...");
  mesg = createSMSsendContent ();
  gtk_widget_show (mesg);
  gtk_notebook_append_page (GTK_NOTEBOOK (subnotebook), mesg, label);
  gtk_widget_show (label);

  gtk_notebook_set_page (GTK_NOTEBOOK (subnotebook), 0);

  return subnotebook;
}

GtkWidget *create_pb_sub_tab () {
  GtkWidget *subtable, *box, *scrolled_window;
  GtkWidget *loadbutton, *deletebutton, *addbutton;
  GtkWidget *phonebookclist = NULL;
  GtkWidget *editbutton, *box2, *vbox;
  GtkWidget *backupbutton, *restorebutton, *vialabel;
  GtkWidget *radio_irda, *radio_bt, *radio_cbl;
  gchar *titles[3] = {"index", "name", "number"};
    
  subtable = gtk_table_new (1, 2, FALSE);
  box = gtk_hbutton_box_new ();
  box2 = gtk_hbox_new (FALSE, 1);
  vbox = gtk_vbox_new (TRUE, 1);

  loadbutton = gtk_button_new_with_label ("load phonebook");
  deletebutton = gtk_button_new_with_label ("delete selected");
  addbutton = gtk_button_new_with_label ("add entry");
  editbutton = gtk_button_new_with_label ("edit entry");
  backupbutton = gtk_button_new_with_label ("backup");
  restorebutton = gtk_button_new_with_label ("restore");

  vialabel = gtk_label_new ("the whole pb via");

  radio_irda = gtk_radio_button_new_with_label (NULL, "irda");
  radio_bt = gtk_radio_button_new_with_label_from_widget (GTK_RADIO_BUTTON (radio_irda), "bluetooth");
  gtk_object_set_data (GTK_OBJECT (radio_irda), "radio_pbbt", radio_bt);
  radio_cbl = gtk_radio_button_new_with_label_from_widget (GTK_RADIO_BUTTON (radio_irda), "serial cable");
  gtk_object_set_data (GTK_OBJECT (radio_irda), "radio_pbcbl", radio_cbl);

  phonebookclist = gtk_clist_new_with_titles (3, titles);
  scrolled_window = gtk_scrolled_window_new (NULL, NULL);
  gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolled_window),
				  GTK_POLICY_AUTOMATIC, GTK_POLICY_ALWAYS);
  gtk_clist_set_selection_mode (GTK_CLIST (phonebookclist), GTK_SELECTION_EXTENDED);
  gtk_clist_set_column_min_width (GTK_CLIST (phonebookclist), 1, 200);

  gtk_container_add (GTK_CONTAINER (scrolled_window), phonebookclist);

  gtk_box_pack_start (GTK_BOX (box), loadbutton, FALSE, FALSE, 2);
  gtk_box_pack_start (GTK_BOX (box), deletebutton, FALSE, FALSE, 2);
  gtk_box_pack_start (GTK_BOX (box), addbutton, FALSE, FALSE, 2);
  gtk_box_pack_start (GTK_BOX (box), editbutton, FALSE, FALSE, 2);

  gtk_box_pack_start (GTK_BOX (box2), backupbutton, FALSE, FALSE, 2);
  gtk_box_pack_start (GTK_BOX (box2), restorebutton, FALSE, FALSE, 2);
  gtk_box_pack_start (GTK_BOX (box2), vialabel, FALSE, FALSE, 2);
  gtk_box_pack_start (GTK_BOX (box2), radio_irda, FALSE, FALSE, 2);
  gtk_box_pack_start (GTK_BOX (box2), radio_bt, FALSE, FALSE, 2);
  gtk_box_pack_start (GTK_BOX (box2), radio_cbl, FALSE, FALSE, 2);

  gtk_box_pack_start (GTK_BOX (vbox), box, FALSE, FALSE, 2);
  gtk_box_pack_start (GTK_BOX (vbox), box2, FALSE, FALSE, 2);

  gtk_table_attach (GTK_TABLE (subtable), vbox, 0, 1, 0, 1,
		    GTK_SHRINK, GTK_SHRINK, 10, 10);
  gtk_table_attach_defaults (GTK_TABLE (subtable), scrolled_window, 0, 1, 1, 2);
  /*
  gtk_widget_show (loadbutton);
  gtk_widget_show (deletebutton);
  gtk_widget_show (addbutton);
  gtk_widget_show (editbutton);
  gtk_widget_show (box);
  gtk_widget_show (phonebookclist);
  gtk_widget_show (scrolled_window);
  */
  gtk_widget_show_all (subtable);

  gtk_signal_connect (GTK_OBJECT (loadbutton), "clicked",
		      GTK_SIGNAL_FUNC (load_phonebook), (gpointer) phonebookclist);

  gtk_signal_connect (GTK_OBJECT (deletebutton), "clicked",
		      GTK_SIGNAL_FUNC (delete_phonebook_entries_ask),
		      (gpointer) phonebookclist);

  gtk_signal_connect (GTK_OBJECT (addbutton), "clicked",
		      GTK_SIGNAL_FUNC (addPBentryDialog),
		      (gpointer) phonebookclist);

  gtk_signal_connect (GTK_OBJECT (editbutton), "clicked",
		      GTK_SIGNAL_FUNC (editPBentryDialog),
		      (gpointer) phonebookclist);

  gtk_signal_connect (GTK_OBJECT (backupbutton), "clicked",
		      GTK_SIGNAL_FUNC (backupPB),
		      (gpointer) radio_irda);

  gtk_signal_connect (GTK_OBJECT (restorebutton), "clicked",
		      GTK_SIGNAL_FUNC (restorePB_ask),
		      (gpointer) radio_irda);

  gtk_signal_connect (GTK_OBJECT (phonebookclist), "click-column",
		      GTK_SIGNAL_FUNC (pbsortfunc), NULL);

  gtk_object_set_data (GTK_OBJECT (mainwindow), "pbclist", phonebookclist);

  return subtable;
}

GtkWidget *create_upload_sub_tab () {
  GtkWidget *frame, *label, *subtable;
  GtkWidget *entry, *name_entry, *button;
  GtkWidget *bdaddress_entry, *channel_entry;
  GtkWidget *radio_irda, *radio_bt, *radio_cbl;
  GtkWidget *hbox1, *hbox2;

  frame = gtk_frame_new ("upload files to phone");
  gtk_container_set_border_width (GTK_CONTAINER (frame), 10);
  gtk_widget_show (frame);

  subtable = gtk_table_new (2, 5, FALSE);
  gtk_container_add (GTK_CONTAINER (frame), subtable);
  gtk_widget_show (subtable);

  label = gtk_label_new ("file:");
  gtk_label_set_justify (GTK_LABEL (label), GTK_JUSTIFY_LEFT);
  gtk_table_attach (GTK_TABLE (subtable), label, 0, 1, 0, 1,
		    GTK_SHRINK, GTK_SHRINK, 10, 10);
  gtk_widget_show (label);

  hbox1 = gtk_hbox_new (FALSE, 1);
  gtk_widget_show (hbox1);

  entry = gtk_entry_new ();
  gtk_box_pack_start (GTK_BOX (hbox1), entry, TRUE, TRUE, 0);
  gtk_signal_connect (GTK_OBJECT (entry), "changed",
		      GTK_SIGNAL_FUNC (filenamechanged),
		      (gpointer) entry);
  gtk_widget_show (entry);

  button = gtk_button_new_with_label ("...");
  gtk_box_pack_start (GTK_BOX (hbox1), button, TRUE, TRUE, 0);
  gtk_table_attach (GTK_TABLE (subtable), hbox1, 1, 2, 0, 1,
		    GTK_FILL | GTK_EXPAND, GTK_SHRINK, 10, 10);
  gtk_signal_connect (GTK_OBJECT (button), "clicked",
		      GTK_SIGNAL_FUNC (createFileSelector),
		      (gpointer) entry);
  gtk_widget_show (button);

  label = gtk_label_new ("name on phone:");
  gtk_label_set_justify (GTK_LABEL (label), GTK_JUSTIFY_LEFT);
  gtk_table_attach (GTK_TABLE (subtable), label, 0, 1, 1, 2,
		    GTK_SHRINK, GTK_SHRINK, 10, 10);
  gtk_widget_show (label);

  name_entry = gtk_entry_new ();
  gtk_table_attach (GTK_TABLE (subtable), name_entry, 1, 3, 1, 2,
		    GTK_EXPAND, GTK_SHRINK, 10, 10);
  gtk_object_set_data (GTK_OBJECT (entry), "nameop", name_entry);
  gtk_widget_show (name_entry);

  label = gtk_label_new ("address (bluetooth only!):");
  gtk_label_set_justify (GTK_LABEL (label), GTK_JUSTIFY_LEFT);
  gtk_table_attach (GTK_TABLE (subtable), label, 0, 1, 2, 3,
		    GTK_SHRINK, GTK_SHRINK, 10, 10);
  gtk_widget_show (label);

  bdaddress_entry = gtk_entry_new ();
  gtk_table_attach (GTK_TABLE (subtable), bdaddress_entry, 1, 3, 2, 3,
		    GTK_EXPAND, GTK_SHRINK, 10, 10);
  gtk_object_set_data (GTK_OBJECT (entry), "bdaddr", bdaddress_entry);
  if (config.bdaddress)
    gtk_entry_set_text (GTK_ENTRY (bdaddress_entry), config.bdaddress);
  gtk_widget_show (bdaddress_entry);

  label = gtk_label_new ("obex push channel (bluetooth only!, usually 10):");
  gtk_label_set_justify (GTK_LABEL (label), GTK_JUSTIFY_LEFT);
  gtk_table_attach (GTK_TABLE (subtable), label, 0, 1, 3, 4,
		    GTK_SHRINK, GTK_SHRINK, 10, 10);
  gtk_widget_show (label);

  channel_entry = gtk_entry_new ();
  gtk_table_attach (GTK_TABLE (subtable), channel_entry, 1, 3, 3, 4,
		    GTK_EXPAND, GTK_SHRINK, 10, 10);
  gtk_object_set_data (GTK_OBJECT (entry), "channel", channel_entry);
  if (config.opchan)
    gtk_entry_set_text (GTK_ENTRY (channel_entry), config.opchan);
  else
    gtk_entry_set_text (GTK_ENTRY (channel_entry), "10");
  gtk_widget_show (channel_entry);

  hbox2 = gtk_hbox_new (FALSE, 10);
  gtk_widget_show (hbox2);

  radio_irda = gtk_radio_button_new_with_label (NULL, "irda");
  gtk_box_pack_start (GTK_BOX (hbox2), radio_irda, TRUE, TRUE, 0);
  gtk_object_set_data (GTK_OBJECT (entry), "radio_irda", radio_irda);
  gtk_widget_show (radio_irda);

  radio_bt = gtk_radio_button_new_with_label_from_widget (GTK_RADIO_BUTTON (radio_irda), "bluetooth");
  gtk_box_pack_start (GTK_BOX (hbox2), radio_bt, TRUE, TRUE, 0);
  gtk_object_set_data (GTK_OBJECT (entry), "radio_bt", radio_bt);
  gtk_widget_show (radio_bt);

  radio_cbl = gtk_radio_button_new_with_label_from_widget (GTK_RADIO_BUTTON (radio_irda), "serial cable");
  gtk_box_pack_start (GTK_BOX (hbox2), radio_cbl, TRUE, TRUE, 0);
  gtk_table_attach (GTK_TABLE (subtable), hbox2, 0, 1, 4, 5,
		    GTK_SHRINK, GTK_SHRINK, 10, 10);
  gtk_object_set_data (GTK_OBJECT (entry), "radio_cbl", radio_cbl);
  gtk_widget_show (radio_cbl);

  button = gtk_button_new_with_label ("upload");
  gtk_table_attach (GTK_TABLE (subtable), button, 1, 2, 4, 5,
		    GTK_SHRINK, GTK_SHRINK, 10, 10);
  gtk_signal_connect (GTK_OBJECT (button), "clicked",
		      GTK_SIGNAL_FUNC (uploadToPhone),
		      (gpointer) entry);
  gtk_widget_show (button);

  return frame;
}

GtkWidget *create_download_sub_tab () {
  GtkWidget *frame;

  frame = gtk_frame_new ("download files to phone");
  gtk_container_set_border_width (GTK_CONTAINER (frame), 10);
  gtk_widget_show (frame);

  return frame;
}

GtkWidget *create_remote_sub_tab () {
  GtkWidget *subtable, *label;
  GtkWidget *ctree, *button;
  GtkWidget *hbox, *vbox;
  GtkCTreeNode *parent;
  gchar *titles[2] = {"name", "command"};
  gchar *root[2] = {"linux box", " "};

  subtable = gtk_table_new (2, 2, FALSE);
  gtk_widget_show (subtable);

  ctree = gtk_ctree_new_with_titles (2, 0, titles);
  gtk_clist_set_column_min_width (GTK_CLIST (ctree), 0, 200);
  parent = gtk_ctree_insert_node (GTK_CTREE (ctree), NULL, NULL, root, 2,
				  NULL, NULL, NULL, NULL, FALSE, FALSE);
  load_remote_list (ctree, parent);

  gtk_table_attach (GTK_TABLE (subtable), ctree, 0, 1, 1, 2,
		    GTK_EXPAND | GTK_FILL, GTK_EXPAND | GTK_FILL, 10, 10);
  gtk_object_set_data (GTK_OBJECT (ctree), "root", (gpointer) parent);
  gtk_signal_connect (GTK_OBJECT (ctree), "destroy",
		      GTK_SIGNAL_FUNC (remote_disconnect), (gpointer) ctree);
  gtk_widget_show (ctree);
    
  hbox = gtk_hbutton_box_new ();
  gtk_table_attach (GTK_TABLE (subtable), hbox, 0, 2, 0, 1,
		    GTK_EXPAND, GTK_SHRINK, 10, 10);
  gtk_widget_show (hbox);

  button = gtk_button_new_with_label ("add entry");
  gtk_box_pack_start (GTK_BOX (hbox), button, FALSE, FALSE, 2);
  gtk_signal_connect (GTK_OBJECT (button), "clicked",
		      GTK_SIGNAL_FUNC (add_remote_entry_dialog),
		      (gpointer) ctree);
  gtk_widget_show (button);

  button = gtk_button_new_with_label ("add command");
  gtk_box_pack_start (GTK_BOX (hbox), button, FALSE, FALSE, 2);
  gtk_signal_connect (GTK_OBJECT (button), "clicked",
		      GTK_SIGNAL_FUNC (add_remote_command_dialog),
		      (gpointer) ctree);
  gtk_widget_show (button);

  button = gtk_button_new_with_label ("edit selected");
  gtk_box_pack_start (GTK_BOX (hbox), button, FALSE, FALSE, 2);
  gtk_signal_connect (GTK_OBJECT (button), "clicked",
		      GTK_SIGNAL_FUNC (edit_remote_entry_dialog),
		      (gpointer) ctree);
  gtk_widget_show (button);

  button = gtk_button_new_with_label ("delete selected");
  gtk_box_pack_start (GTK_BOX (hbox), button, FALSE, FALSE, 2);
  gtk_signal_connect (GTK_OBJECT (button), "clicked",
		      GTK_SIGNAL_FUNC (delete_remote_entry_dialog),
		      (gpointer) ctree);
  gtk_widget_show (button);

  button = gtk_button_new_with_label ("save list");
  gtk_box_pack_start (GTK_BOX (hbox), button, FALSE, FALSE, 2);
  gtk_signal_connect (GTK_OBJECT (button), "clicked",
		      GTK_SIGNAL_FUNC (save_remote_list),
		      (gpointer) ctree);
  gtk_widget_show (button);

  vbox = gtk_vbutton_box_new ();
  gtk_table_attach (GTK_TABLE (subtable), vbox, 1, 2, 0, 2,
		    GTK_SHRINK, GTK_SHRINK, 0, 0);
  gtk_widget_show (vbox);

  button = gtk_button_new_with_label ("connect");
  gtk_box_pack_start (GTK_BOX (vbox), button, FALSE, FALSE, 2);
  gtk_signal_connect (GTK_OBJECT (button), "clicked",
		      GTK_SIGNAL_FUNC (remote_connect),
		      (gpointer) ctree);
  gtk_widget_show (button);
  /*
  button = gtk_button_new_with_label ("disconnect");
  gtk_box_pack_start (GTK_BOX (vbox), button, FALSE, FALSE, 2);
  gtk_signal_connect (GTK_OBJECT (button), "clicked",
		      GTK_SIGNAL_FUNC (remote_disconnect),
		      (gpointer) ctree);
  gtk_widget_show (button);
  */

  return subtable;
}

GtkWidget *create_config_sub_tab () {
  GtkWidget *frame, *label, *button;
  GtkWidget *subtable, *entry;

  frame = gtk_frame_new ("configuration");
  gtk_container_set_border_width (GTK_CONTAINER (frame), 10);
  gtk_widget_show (frame);

  subtable = gtk_table_new (2, 4, FALSE);
  gtk_container_add (GTK_CONTAINER (frame), subtable);
  gtk_widget_show (subtable);

  label = gtk_label_new ("device:");
  gtk_label_set_justify (GTK_LABEL (label), GTK_JUSTIFY_LEFT);
  gtk_table_attach (GTK_TABLE (subtable), label, 0, 1, 0, 1,
		    GTK_SHRINK, GTK_SHRINK, 10, 10);
  gtk_widget_show (label);

  button = gtk_button_new_with_label ("save");

  entry = gtk_entry_new ();
  gtk_table_attach (GTK_TABLE (subtable), entry, 1, 2, 0, 1,
		    GTK_FILL | GTK_EXPAND, GTK_SHRINK, 10, 10);
  if (config.device != NULL)
    gtk_entry_set_text (GTK_ENTRY (entry), config.device);
  gtk_object_set_data (GTK_OBJECT (button), "device", entry);
  gtk_widget_show (entry);

#ifdef HAVE_BLUETOOTH
  label = gtk_label_new ("bluetooth address:");
  gtk_label_set_justify (GTK_LABEL (label), GTK_JUSTIFY_LEFT);
  gtk_table_attach (GTK_TABLE (subtable), label, 0, 1, 1, 2,
		    GTK_SHRINK, GTK_SHRINK, 10, 10);
  gtk_widget_show (label);

  entry = gtk_entry_new ();
  gtk_table_attach (GTK_TABLE (subtable), entry, 1, 2, 1, 2,
		    GTK_FILL | GTK_EXPAND, GTK_SHRINK, 10, 10);
  if (config.bdaddress != NULL)
    gtk_entry_set_text (GTK_ENTRY (entry), config.bdaddress);
  gtk_object_set_data (GTK_OBJECT (button), "address", entry);
  gtk_widget_show (entry);

  label = gtk_label_new ("obex push port:");
  gtk_label_set_justify (GTK_LABEL (label), GTK_JUSTIFY_LEFT);
  gtk_table_attach (GTK_TABLE (subtable), label, 0, 1, 2, 3,
		    GTK_SHRINK, GTK_SHRINK, 10, 10);
  gtk_widget_show (label);

  entry = gtk_entry_new ();
  gtk_table_attach (GTK_TABLE (subtable), entry, 1, 2, 2, 3,
		    GTK_FILL | GTK_EXPAND, GTK_SHRINK, 10, 10);
  gtk_object_set_data (GTK_OBJECT (button), "chan", entry);
  if (config.opchan != NULL)
    gtk_entry_set_text (GTK_ENTRY (entry), config.opchan);
  gtk_widget_show (entry);
#endif

  gtk_table_attach (GTK_TABLE (subtable), button, 1, 2, 3, 4,
		    GTK_SHRINK, GTK_SHRINK, 10, 10);
  gtk_signal_connect (GTK_OBJECT (button), "clicked",
		      GTK_SIGNAL_FUNC (save_config), NULL);
  gtk_widget_show (button);

  return frame;
}

GtkWidget *create_about_sub_tab () {
  GtkWidget *frame, *label, *subtable;
  GtkWidget *pixmapwid;
  gchar *tmp;

  tmp = g_strdup_printf ("about tsemgr %s", PACKAGE_VERSION);
  frame = gtk_frame_new (tmp);
  gtk_container_set_border_width (GTK_CONTAINER (frame), 10);
  gtk_widget_show (frame);

  subtable = gtk_table_new (2, 1, FALSE);
  gtk_container_add (GTK_CONTAINER (frame), subtable);
  gtk_widget_show (subtable);
        
  pixmapwid = createPixmap (xpm_phone);
  gtk_widget_show (pixmapwid);

  g_free (tmp);
  tmp = g_strdup_printf (abouttext);
  label = gtk_label_new (tmp);
  gtk_label_set_justify (GTK_LABEL (label), GTK_JUSTIFY_LEFT);
  gtk_table_attach_defaults (GTK_TABLE (subtable), label, 1, 2, 0, 1);
  gtk_table_attach_defaults (GTK_TABLE (subtable), pixmapwid, 0, 1, 0, 1);
  gtk_widget_show (label);
    
  g_free (tmp);

  return frame;
}

void my_gtk_notebook_append_tab (GtkWidget *notebook, GtkWidget *child,
				 gchar *tab_label, char *pixmap[]) {
  GtkWidget *label;
  GtkWidget *hbox;
  GtkWidget *pixmapwid;

  pixmapwid = createPixmap (pixmap);
  hbox = gtk_vbox_new (FALSE, 1);
  label = gtk_label_new (tab_label);

  gtk_widget_show (pixmapwid);
  gtk_widget_show (hbox);
  gtk_widget_show (label);

  gtk_box_pack_start (GTK_BOX (hbox), pixmapwid, TRUE, TRUE, 0);
  gtk_box_pack_start (GTK_BOX (hbox), label, TRUE, TRUE, 0);

  gtk_notebook_append_page (GTK_NOTEBOOK (notebook), child, hbox);
}

GtkWidget *createTabs () {
  GtkWidget *notebook;
  GtkWidget *table;
  GtkWidget *label;
  GtkWidget *button;
  GtkWidget *button_box;
  GtkWidget *hbox;
  GtkWidget *pixmapwid;
  GtkWidget *child;

  table = gtk_table_new (1, 2, FALSE);

  notebook = gtk_notebook_new ();
  gtk_notebook_set_tab_pos (GTK_NOTEBOOK (notebook), GTK_POS_TOP);
  gtk_table_attach_defaults (GTK_TABLE (table), notebook, 0, 2, 2, 3);
    
  /* SMS TAB */
  child = create_sms_sub_tab ();
  my_gtk_notebook_append_tab (notebook, child, "SMS", xpm_sms);
  /* SMS TAB */

  /* PHONEBOOK TAB */
  child = create_pb_sub_tab ();
  my_gtk_notebook_append_tab (notebook, child, "PhoneBook", xpm_phonebook);
  /* PHONEBOOK TAB */

  /* UPLOAD TAB */
  child = create_upload_sub_tab ();
  my_gtk_notebook_append_tab (notebook, child, "Upload", xpm_upload);
  /* UPLOAD TAB */

  /* DOWNLOAD TAB */
  /*
  child = create_download_sub_tab ();
  my_gtk_notebook_append_tab (notebook, child, "Download", xpm_upload);
  */
  /* DOWNLOAD TAB */

  /* REMOTE TAB */
  child = create_remote_sub_tab ();
  my_gtk_notebook_append_tab (notebook, child, "Remote", xpm_remote);
  /* REMOTE TAB */

  /* CONFIG TAB */
  child = create_config_sub_tab ();
  my_gtk_notebook_append_tab (notebook, child, "Config", xpm_config);
  /* CONFIG TAB */

  /* ABOUT TAB */
  child = create_about_sub_tab ();
  my_gtk_notebook_append_tab (notebook, child, "About", xpm_about);
  /* ABOUT TAB */

  gtk_notebook_set_show_tabs (GTK_NOTEBOOK (notebook), TRUE);
  gtk_notebook_set_show_border (GTK_NOTEBOOK (notebook), FALSE);
  gtk_notebook_set_page (GTK_NOTEBOOK (notebook), 6);
  gtk_notebook_set_scrollable (GTK_NOTEBOOK (notebook), TRUE);
  gtk_notebook_set_tab_border (GTK_NOTEBOOK (notebook), 2);
  gtk_notebook_set_homogeneous_tabs (GTK_NOTEBOOK (notebook), TRUE);

  gtk_widget_show (notebook);
  return table;
}

GtkWidget *displaySMS (recievedSMS *sms) {
  GtkWidget *table;
  GtkWidget *label;
  gchar *number, *date, *tmp;
  gint i;

  table = gtk_table_new (2, 2, TRUE);

  number = g_strdup_printf ("\nSenderNumber:\n%s\n",
			    (gchar *) sms->senderNumber);
  label = gtk_label_new (number);
  gtk_label_set_justify (GTK_LABEL (label), GTK_JUSTIFY_LEFT);
  gtk_table_attach (GTK_TABLE (table), label, 0, 1, 0, 1,
		    GTK_FILL, GTK_FILL, 10, 0);
  gtk_misc_set_alignment (GTK_MISC (label), 0, 0.5);
  gtk_widget_show (label);
  gtk_object_set_data (GTK_OBJECT (table), "number", label);

  date = g_strdup_printf ("\nDate:\n%s\n", (gchar *) sms->timeStamp);
  label = gtk_label_new (date);
  gtk_label_set_justify (GTK_LABEL (label), GTK_JUSTIFY_LEFT);
  gtk_table_attach (GTK_TABLE (table), label, 1, 2, 0, 1,
		    GTK_FILL, GTK_FILL, 10, 0);
  gtk_misc_set_alignment (GTK_MISC (label), 0, 0.5);
  gtk_widget_show (label);
  gtk_object_set_data (GTK_OBJECT (table), "date", label);


  tmp = g_strdup (sms->message);
  if (sms->udh != 0) {
    if (sms->pda != 0)
      for (i = 0; i < sms->pda; i++)
	tmp = insertElement ("<predefined animation>",
			     sms->message, sms->pda_pos[i]);
    if (sms->la != 0)
      for (i = 0; i < sms->la; i++)
	tmp = insertElement ("<large animation>",
			     sms->message, sms->la_pos[i]);
    if (sms->sa != 0)
      for (i = 0; i < sms->sa; i++)
	tmp = insertElement ("<small animation>",
			     sms->message, sms->sa_pos[i]);
    if (sms->lp != 0)
      for (i = 0; i < sms->lp; i++)
	tmp = insertElement ("<large picture>",
			     sms->message, sms->lp_pos[i]);
    if (sms->sp != 0)
      for (i = 0; i < sms->sp; i++)
	tmp = insertElement ("<small picture>",
			     sms->message, sms->sp_pos[i]);
    if (sms->vp != 0)
      for (i = 0; i < sms->vp; i++)
	tmp = insertElement ("<variable picture>",
			     sms->message, sms->vp_pos[i]);
    if (sms->pds != 0)
      for (i = 0; i < sms->pds; i++)
	tmp = insertElement ("<predefined sound>",
			     sms->message, sms->pds_pos[i]);
    if (sms->uds != 0)
      for (i = 0; i < sms->uds; i++)
	tmp = insertElement ("<user defined sound>",
			     sms->message, sms->uds_pos[i]);
    if (sms->concat != 0)
      tmp = g_strdup_printf ("%s\n\nmessage %d of %d\nreference: %d",
			     tmp, sms->concat_cur, sms->concat_max,
			     sms->concat_reference);
  }

  label = gtk_label_new (tmp);
  gtk_label_set_line_wrap (GTK_LABEL (label), TRUE);
  gtk_label_set_justify (GTK_LABEL (label), GTK_JUSTIFY_LEFT);
  gtk_table_attach (GTK_TABLE (table), label, 0, 2, 1, 2,
		    GTK_FILL, GTK_FILL, 10, 0);
  gtk_misc_set_alignment (GTK_MISC (label), 0, 0.5);
  gtk_widget_show (label);
  gtk_object_set_data (GTK_OBJECT (table), "message", label);

  g_free (tmp);
  g_free (number);
  g_free (date);

  return table;
}

void displaySMSList (GtkWidget *widget, gpointer *scrolled_window, gint memtype) {
  GtkWidget *dialog, *progress_bar;
  GtkWidget *subtable;
  GtkWidget *table;
  GtkWidget *savebutton;
  GtkWidget *separator;
  GtkWidget *checkbutton;
  GtkWidget *box;
  GSList *mylist = NULL;
  recievedSMS *mysms = NULL;
  gchar *key;
  gchar *indexstr;
  gint index;
  gint i = 0;

  if (gtk_object_get_data (GTK_OBJECT (scrolled_window), "table") != 0) {
    table = gtk_object_get_data (GTK_OBJECT (scrolled_window), "table");
    gtk_widget_destroy (table);
  }
  table = gtk_table_new (2, 1, FALSE);
  gtk_scrolled_window_add_with_viewport (GTK_SCROLLED_WINDOW
					 (scrolled_window), table);
  gtk_object_set_data (GTK_OBJECT (scrolled_window), "table", table);

  dialog = SMSstatusDialog ();
  gtk_widget_show_all (dialog);
  progress_bar = gtk_object_get_data (GTK_OBJECT (dialog), "progress_bar");
  while (gtk_events_pending ())
    gtk_main_iteration();
  for (mylist = listSMS (memtype, 1, progress_bar); mylist != NULL;
       mylist = g_slist_next (mylist)) {
    mysms = parseSMS (mylist->data + 2);
    indexstr = g_strndup (mylist->data, 2);
    index = hexstr2int (indexstr, 2);
    subtable = displaySMS (mysms);
    gtk_widget_show (subtable);

    gtk_table_resize (GTK_TABLE (table), 2, i + 1);
    gtk_table_attach_defaults (GTK_TABLE (table), subtable, 0, 1, i, i + 1);

    separator = gtk_hseparator_new ();
    gtk_widget_show (separator);

    box = gtk_vbox_new (FALSE, 10);
    gtk_widget_show (box);
    savebutton = gtk_button_new_with_label ("save");
    gtk_widget_show (savebutton);
    gtk_container_add (GTK_CONTAINER (box), savebutton);
    gtk_button_set_relief (GTK_BUTTON (savebutton), GTK_RELIEF_NONE);
    gtk_signal_connect (GTK_OBJECT (savebutton), "clicked",
			GTK_SIGNAL_FUNC (saveSMS),
			(gpointer) subtable);

    checkbutton = gtk_check_button_new ();
    gtk_widget_show (checkbutton);
    gtk_container_add (GTK_CONTAINER (box), checkbutton);

    gtk_table_attach (GTK_TABLE (table), box, 1, 2, i, i + 1,
		      GTK_SHRINK, GTK_SHRINK, 0, 0);

    gtk_table_attach_defaults (GTK_TABLE (table), separator,
			       0, 2, i + 1, i + 2);
    g_free (mysms);
    key = g_strdup_printf ("checkbutton%d", i / 2);
    gtk_object_set_data (GTK_OBJECT (scrolled_window), key, (gpointer) checkbutton);
    g_free (key);
    key = g_strdup_printf ("subtable%d", i / 2);
    gtk_object_set_data (GTK_OBJECT (scrolled_window), key, (gpointer) subtable);
    g_free (key);
    key = g_strdup_printf ("index%d", i / 2);
    gtk_object_set_data (GTK_OBJECT (scrolled_window), key,
			 GINT_TO_POINTER (index));
    g_free (key);
    gtk_object_set_data (GTK_OBJECT (scrolled_window), "memtype",
			 GINT_TO_POINTER (memtype));
    i += 2;
  }
  gtk_object_set_data (GTK_OBJECT (scrolled_window), "total_checkbuttons",
		       GINT_TO_POINTER (i));
  gtk_widget_show (table);
  gtk_widget_destroy (dialog);
  g_slist_free (mylist);
}

