/*
  tsemgr is a gtk application to manage a sonyericsson t68 mobile phone
  Copyright (C) 2003 Sebastian Sareyko

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/


#include "config.h"
#include <gtk/gtk.h>
#include <ezV24/ezV24.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <stdio.h>
#include <stdlib.h>
#include "tsemgr.h"
#include "phonebook.h"
#include "interface.h"

/* 
   hexstr2int was taken from the scmxx-0.6.3.5
   (which was made by Hendrik Sattler) source, and
   then modified to my needs!
   go get it at: http://www.hendrik-sattler.de/scmxx/
*/
gint hexstr2int (gchar *hexstring, gint length){
  gint i, shift;
  gint digitvalue = 0;
  gint intvalue = 0;
  gchar hexdigit;
	
  // simply believe me that atoi DID NOT WORK RELIABLE
  for (i = 0; i < length; i++){
    switch (hexdigit = hexstring[i]){
    case '1':
      digitvalue = 1;
      break;
    case '2':
      digitvalue = 2;
      break;
    case '3':
      digitvalue = 3;
      break;
    case '4':
      digitvalue = 4;
      break;
    case '5':
      digitvalue = 5;
      break;
    case '6':
      digitvalue = 6;
      break;
    case '7':
      digitvalue = 7;
      break;
    case '8':
      digitvalue = 8;
      break;
    case '9':
      digitvalue = 9;
      break;
    case 'A':
    case 'a':
      digitvalue = 10;
      break;
    case 'B':
    case 'b':
      digitvalue = 11;
      break;
    case 'C':
    case 'c':
      digitvalue = 12;
      break;
    case 'D':
    case 'd':
      digitvalue = 13;
      break;
    case 'E':
    case 'e':
      digitvalue = 14;
      break;
    case 'F':
    case 'f':
      digitvalue = 15;
      break;
    case '0':
    default:
      digitvalue = 0;
      break;
    }
    shift = 4 * (length - i - 1);  // multiplicator: <<4*  = *16^
    intvalue += digitvalue << shift;
  }
  return (intvalue);
}

void load_config_file () {
  gchar line[1024];
  gchar *tmp;
  FILE *fd;

  config.device = NULL;
  config.bdaddress = NULL;
  config.opchan = NULL;
  if (config_base_dir != NULL) {
    tmp = g_strdup_printf ("%s/%s", config_base_dir, BASE_CONFIG_FILE);
    if ((fd = fopen (tmp, "r")) != NULL) {
      fgets (line, sizeof (line), fd);
      config.device = g_strdup (line);
      g_strdelimit (config.device, "\n", '\0');
#ifdef HAVE_BLUETOOTH
      fgets (line, sizeof (line), fd);
      config.bdaddress = g_strdup (line);
      g_strdelimit (config.bdaddress, "\n", '\0');
      fgets (line, sizeof (line), fd);
      config.opchan = g_strdup (line);
      g_strdelimit (config.opchan, "\n", '\0');
#endif
      fclose (fd);
    }
    g_free (tmp);
  }
}

gchar *check_base_config_dir () {
  gchar *bdir;

  bdir = getenv ("HOME");
  if (bdir != NULL) {
    bdir = g_strdup_printf ("%s/%s", bdir, TSEMGR_BASE_CONFIG_DIR);
    mkdir (bdir, 0777);
    return bdir;
  }
  else
    return NULL;
}

struct t68properties store_properties () {
  GtkWidget *dialog;
  struct t68properties props;
  v24_port_t *port = NULL;

  dialog = messageDialog ("initializing...\nplease wait", 0);

  while (gtk_events_pending ())
    gtk_main_iteration ();
  port = v24OpenPort (config.device, V24_RTS_CTS);
  if (port != NULL) {
    getPBEntryLen (port, &props);
    v24ClosePort (port);
  }
  else {
    props.pbnamelen = 20;
    props.pbnumberlen = 20;
    errorDialog ("unable to open device!\nplease check if your phone is connected\nand if you specified the correct device!");
  }

  gtk_widget_destroy (dialog);

  return props;
}

gchar *insertElement (gchar *text, gchar *message, gint pos) {
  gchar *tmp1, *tmp2, *tmp;

  if (pos)
    tmp1 = g_strndup (message, pos);
  else
    tmp1 = g_strdup ("");
  if (strlen (message) - pos > 0)
    tmp2 = g_strndup (message + pos, strlen (message) - pos);
  else
    tmp2 = g_strdup ("");
  tmp = g_strdup_printf ("%s%s%s", tmp1, text, tmp2);

  g_free (tmp1);
  g_free (tmp2);

  return tmp;
}
