/*
  tsemgr is a gtk application to manage a sonyericsson t68 mobile phone
  Copyright (C) 2003 Sebastian Sareyko

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/


#include "config.h"
#include <gtk/gtk.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <errno.h>
#include <unistd.h>
#include <stdint.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include "obex_misc.h"
#include "interface.h"
#include "getfile.h"

void save_obex_file (gchar *name, const uint8_t *buf, gint len) {
  gchar *s = NULL;
  gchar *filename;
  gint fd;

  if (config_base_dir != NULL) {
    s = strrchr (name, '/');
    if (s == NULL)
      s = name;
    else
      s++;

    filename = g_strdup_printf ("%s/%s", config_base_dir, s);
    fd = creat (filename, 0666);
    if (fd != -1) {
      ftruncate (fd, 0);
      if (write (fd, buf, len) == -1)
	errorDialog ("cannot write to file!");
      close (fd);
    }
    else
      errorDialog ("cannot save to file!\nunable to create file");
    g_free (filename);
  }
  else
    errorDialog ("cannot save to file!!\nreason:\nyour $HOME variable is not set\ndo export HOME=/pathto/yourhomedir\nand start tsemgr again");
}

gint get_filesize (const gchar *filename) {
  struct stat stats;

  stat (filename, &stats);
  return (int) stats.st_size;
}

uint8_t* easy_readfile (const gchar *filename, gint *file_size) {
  gint actual;
  gint fd;
  uint8_t *buf;

  *file_size = get_filesize (filename);
  //  printf ("name=%s, size=%d\n", filename, *file_size);

  fd = open (filename, O_RDONLY);

  if (fd == -1) {
    return NULL;
  }
	
  if (!(buf = malloc (*file_size))) {
    return NULL;
  }

  actual = read (fd, buf, *file_size);
  close (fd); 

  *file_size = actual;

  return buf;
}

gint t68_connect (obex_t *handle) {
  obex_object_t *object;
  obex_headerdata_t hd;	

  /* Create a new obex object request */
  if(!(object = OBEX_ObjectNew (handle, OBEX_CMD_CONNECT))) {
    fprintf(stderr,"Can't create an obexobject\n");
    return -1;
  }

  hd.bs = "Linux";
  if(OBEX_ObjectAddHeader(handle, object, OBEX_HDR_WHO, hd, 6,
			  OBEX_FL_FIT_ONE_PACKET) < 0) {
    fprintf(stderr,"Error adding header\n");
    OBEX_ObjectDelete(handle, object);
    return -1;
  }
  OBEX_Request(handle, object);
  syncwait(handle);
	
  //  fprintf(stderr,"Connected\n");
	
  return 0;
}

gint t68_disconnect (obex_t* handle) {
  //  fprintf (stderr,"Disconnected\n");
  OBEX_TransportDisconnect (handle);
  /*"check the return value of the above!"*/
  return 0;
}

void obex_event (obex_t *handle, obex_object_t *object, gint mode, gint event, gint obex_cmd, gint obex_rsp) {

  while (gtk_events_pending())
    gtk_main_iteration();
  switch (event)	{
  case OBEX_EV_PROGRESS:
    printf ("Made some progress...\n");
    break;

  case OBEX_EV_ABORT:
    printf ("Request aborted!\n");
    errorDialog ("request aborted!");
    break;

  case OBEX_EV_REQDONE:
    if (mode == OBEX_CLIENT)
      client_done (handle, object, obex_cmd, obex_rsp);
    break;
  case OBEX_EV_REQHINT:
    /* Accept any command. Not rellay good, but this is a test-program :) */
    OBEX_ObjectSetRsp (object, OBEX_RSP_CONTINUE, OBEX_RSP_SUCCESS);
    break;

  case OBEX_EV_REQ:
    //    server_request(handle, object, event, obex_cmd);
    break;

  case OBEX_EV_LINKERR:
    OBEX_TransportDisconnect (handle);
    printf ("Link broken!\n");
    errorDialog ("link broken!");
    break;

  case OBEX_EV_STREAMEMPTY:
    //    fillstream(handle, object);
    break;

  case OBEX_EV_UNEXPECTED:
    break;
  default:
    printf ("Unknown event %02x!\n", event);
    break;
  }
}

void client_done (obex_t *handle, obex_object_t *object, gint obex_cmd, gint obex_rsp) {
  struct context *gt;

  gt = OBEX_GetUserData(handle);

  switch (obex_cmd) {
  case OBEX_CMD_CONNECT:
    if (obex_rsp != OBEX_RSP_SUCCESS) {
      //      g_print ("connected\n");
      //    else
      g_print ("connect failed\n");
      errorDialog ("connect failed!");
    }
    break;
  case OBEX_CMD_PUT:
    if (obex_rsp == OBEX_RSP_SUCCESS) {
      //      printf ("PUT successful!\n");
    }	
    else {
      printf ("PUT failed 0x%02x!\n", obex_rsp);
      errorDialog ("sending file failed!");
    }
    break;
  case OBEX_CMD_GET:
    client_done_get (handle, object, obex_rsp, gt->get_name);
    break;
  case OBEX_CMD_DISCONNECT:
    object = OBEX_ObjectNew (handle, OBEX_CMD_DISCONNECT);
    OBEX_Request (handle, object);
    syncwait (handle);
    break;
  }
  gt->clientdone = TRUE;
}

void syncwait (obex_t *handle) {
  struct context *gt;
  gint ret;
	
  while (gtk_events_pending())
    gtk_main_iteration();
  gt = OBEX_GetUserData (handle);

  while (!gt->clientdone) {
    ret = OBEX_HandleInput (handle, 10);
    if (ret < 0) {
      printf ("Error while doing OBEX_HandleInput()\n");
      break;
    }
    if (ret == 0) {
      /* If running cable. We get no link-errors, so cancel on timeout */
      printf ("Timeout waiting for data. Aborting\n");
      OBEX_CancelRequest (handle, FALSE);
      break;
    }
  }

  gt->clientdone = FALSE;
}
