/*
  tsemgr is a gtk application to manage a sonyericsson t68 mobile phone
  Copyright (C) 2003 Sebastian Sareyko

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/


#include <gtk/gtk.h>
#include <string.h>
#include "config.h"
#include "phonebook.h"
//#include "sms.h"
#include "interface.h"
#include "callbacks.h"

gint pNumber (gchar *number) {
  gint i;
  gchar *curnumber;
  const gchar *allowed = "0123456789+";

  if (strlen (number) < 1)
    return 0;
  for (i = 0; i <= strlen (number); i++) {
    curnumber = g_strdup_printf ("%c", number[i]);
    if (strstr (allowed, curnumber) == NULL) {
      g_free (curnumber);
      return 0;
    }
  }
  g_free (curnumber);
  return 1;
}

gint selectPBstorage (v24_port_t *port, gchar *storage) {
  gchar *command;
  gchar buffer[512];
  gint stop = 0;
  gint toReturn = 1;

  command = g_strdup_printf ("AT+CPBS=\"%s\"\r", storage);
  v24Puts (port, command);
  while (!stop) {
    while (gtk_events_pending ())
      gtk_main_iteration ();
    v24Gets (port, buffer, 512);
    if (strstr (buffer, "OK"))
      stop = 1;
    else if (strstr (buffer, "ERROR")) {
      stop = 1;
      toReturn = 0;
    }
  }
  return toReturn;
}

gint writePBentry (gint index, gchar *number, gchar *name) {
  gchar *command;
  gchar buffer[512];
  gint stop = 0;
  gint toReturn = 1;
  v24_port_t *port = NULL;

  if ((strcasecmp (number, "") == 0) && (strcasecmp (name, "") == 0) && (index != 0))
    command = g_strdup_printf ("AT+CPBW=%d\r", index);
  else if (index == 0)
    command = g_strdup_printf ("AT+CPBW=,\"%s\",,\"%s\"\r", number, name);
  else if (index != 0)
    command = g_strdup_printf ("AT+CPBW=%d,\"%s\",,\"%s\"\r", index, number, name);
  port = v24OpenPort (config.device, V24_STANDARD);
  if (port != NULL) {
    selectPBstorage (port, "ME");
    v24Puts (port, command);
    while (!stop) {
      while (gtk_events_pending())
	gtk_main_iteration();
      v24Gets (port, buffer, 512);
      if (strstr (buffer, "OK"))
	stop = 1;
      else if (strstr (buffer, "ERROR")) {
	stop = 1;
	toReturn = 0;
      }
    }
    v24ClosePort (port);
  }
  else {
    toReturn = 0;
    errorDialog ("unable to open device!\nplease check if your phone is connected\nand if you specified the correct device!");
  }
  return toReturn;
}

void getPBEntryLen (v24_port_t *port, struct t68properties *props) {
  gchar buffer[512];
  gchar **entry;
  gint stop = 0;

  selectPBstorage (port, "ME");

  v24Puts (port, "AT+CPBR=?\r");
  v24Gets (port, buffer, 512);
  while (!stop) {
    v24Gets (port, buffer, 512);
    if (strstr (buffer, "+CPBR:")) {
      entry = g_strsplit (buffer + 6, ",", 3);
      props->pbnamelen = atoi (entry[2]);
      props->pbnumberlen = atoi (entry[1]);
      g_strfreev (entry);
      stop = 1;
    }
    else if (strstr (buffer, "ERROR")) {
      props->pbnamelen = 20;
      props->pbnumberlen = 20;
      stop = 1;
    }
  }
}
