/*
  tsemgr is a gtk application to manage a sonyericsson t68 mobile phone
  Copyright (C) 2003 Sebastian Sareyko

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

#include <gtk/gtk.h>
#include <ezV24/ezV24.h>
#include <string.h>
#include <unistd.h>
#include <errno.h>
#include <signal.h>
#include <sys/wait.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <stdio.h>
#include "remote.h"
#include "config.h"
#include "tsemgr.h"

void load_remote_list (GtkWidget *ctree, GtkCTreeNode *root) {
  GtkCTreeNode *node, *parent;
  gchar line[1024];
  gchar **splitstr;
  gchar *nodestr[2];
  gchar *tmp;
  gint nodetype;
  FILE *fd;

  if (config_base_dir != NULL) {
    tmp = g_strdup_printf ("%s/%s", config_base_dir, REMOTE_CTREE_FILE);
    parent = root;
    if ((fd = fopen (tmp, "r")) != NULL) {
      while (fgets (line, sizeof (line), fd)) {
	splitstr = g_strsplit (line, ",", 4);
	nodetype = atoi (splitstr[0]);
	g_strstrip (splitstr[3]);
	nodestr[0] = splitstr[2];
	nodestr[1] = splitstr[3];
	if (strstr (splitstr[1], "root")) {
	  parent = gtk_ctree_insert_node (GTK_CTREE (ctree), root, NULL, nodestr,
					  2, NULL, NULL, NULL, NULL, FALSE, FALSE);
	  gtk_ctree_node_set_row_data (GTK_CTREE (ctree), parent,
				       (gpointer) nodetype);
	}
	else {
	  node = gtk_ctree_insert_node (GTK_CTREE (ctree), parent, NULL, nodestr, 2,
					NULL, NULL, NULL, NULL, FALSE, FALSE);
	  gtk_ctree_node_set_row_data (GTK_CTREE (ctree), node,
				       (gpointer) nodetype);
	}
	g_strfreev (splitstr);
      }
      fclose (fd);
    }
    g_free (tmp);
  }
}

void mySignalHandlerHUP (int reason) {
  _exit (0);
}

void mySignalHandler (int reason) {
  wait (NULL);
}

gint runcmd (char *cmd, char *argv[]) {
  pid_t pid;
  int status;

  pid = fork ();

  if (pid == 0) {
    setpgid (getpid (), getppid ());
    status = execvp (cmd, argv);
    g_print ("%d\n", status);
    _exit (0);
  }
  else
    signal (SIGCHLD, mySignalHandler);

  return 1;
}

void create_accessory_menus_and_wait (gpointer ctree) {
  GtkCTreeNode *root, *node, *nextnode, *prevnode;
  gchar *temp;
  gchar *tmp = NULL;
  guint8 spacing;
  GdkPixmap *pixmap;
  GdkBitmap *mask;
  gchar buffer[512];
  gchar *command = NULL;
  gchar *callstr = NULL;
  gchar *parentname = NULL;
  gint stop = 0;
  gint i = 0;
  gint depth = 1;
  gint k = 0;
  gint call;
  v24_port_t *port = NULL;
  gchar **args = NULL;
  char *argv[200] = {NULL};
  remoteMenuStructure *mymenu;
  pid_t pid;

  mymenu = g_malloc (sizeof (remoteMenuStructure));

  port = gtk_object_get_data (GTK_OBJECT (ctree), "v24port");

  root = gtk_object_get_data (GTK_OBJECT (ctree), "root");
  gtk_ctree_expand_recursive (GTK_CTREE (ctree), NULL);

  //  memset (nodetype, 0, sizeof (nodetype));

  mymenu->mainmenu = g_strdup ("");
  mymenu->submenu[0] = g_strdup ("");

  node = GTK_CTREE_NODE_NEXT (root);

  //  temp = g_malloc (sizeof (gchar *));

  while (node != NULL) {
    nextnode = GTK_CTREE_NODE_NEXT (node);
    prevnode = GTK_CTREE_NODE_PREV (node);
    gtk_ctree_node_get_pixtext (GTK_CTREE (ctree), node, 0, &temp,
				&spacing, &pixmap, &mask);
    tmp = g_strdup (temp);
    if (GTK_CTREE_ROW (node)->parent == root) {
      ++i;
      mymenu->dpth[i] = 1;
      mymenu->column1[i][1] = tmp;
      mymenu->nodetype[i][1] = (gint) gtk_ctree_node_get_row_data (GTK_CTREE
								   (ctree), node);
      if (mymenu->nodetype[i][1] == 1) {
	gtk_ctree_node_get_text (GTK_CTREE (ctree), node, 1, &temp);
	tmp = g_strdup (temp);
	mymenu->column2[i][1] = tmp;
      }
      else {
	mymenu->column2[i][1] = g_strdup ("");
	mymenu->dpth[i] += depth;
      }
      mymenu->mainmenu = g_strdup_printf ("%s,\"%s\"", mymenu->mainmenu,
					  mymenu->column1[i][1]);
    }
    else {
      if (prevnode == GTK_CTREE_ROW (node)->parent) {
	++depth;
	k = 1;
      }
      else
	++k;
      mymenu->column1[k][depth] = tmp;
      mymenu->nodetype[k][depth] = (gint) gtk_ctree_node_get_row_data (GTK_CTREE (ctree), node);
      if (mymenu->nodetype[k][depth] == 1) {
	gtk_ctree_node_get_text (GTK_CTREE (ctree), node, 1, &temp);
	tmp = g_strdup (temp);
	mymenu->column2[k][depth] = tmp;
      }
      else
	mymenu->column2[k][depth] = g_strdup ("");
      mymenu->submenu[0] = g_strdup_printf ("%s,\"%s\"", mymenu->submenu[0],
					    mymenu->column1[k][depth]);
      if ((nextnode == NULL) ||
	  (GTK_CTREE_ROW (nextnode)->parent != GTK_CTREE_ROW (node)->parent)) {
	gtk_ctree_node_get_pixtext (GTK_CTREE (ctree),
				    GTK_CTREE_ROW (node)->parent, 0,
				    &parentname, &spacing, &pixmap, &mask);
	mymenu->submenu[i] = g_strdup_printf ("\"%s\",0,1,%d%s", parentname,
					      k, mymenu->submenu[0]);
	mymenu->submenu[0] = g_strdup ("");
      }
    }
    node = nextnode;
  }


  mymenu->mainmenu = g_strdup_printf ("\"linux box\",0,1,%d%s", i,
				      mymenu->mainmenu);
  i = 0;

  v24Puts (port, "AT*EAM=\"linux box\"\r");

  pid = fork ();
  if (pid == 0) {
    setpgid (getpid (), 0);
    signal (SIGHUP, mySignalHandlerHUP);
    while (stop != -1) {
      stop = v24Gets (port, buffer, 512);
      //    g_print ("%s\n", buffer);
      if (strstr (buffer, "EAAI")) {
	command = g_strdup_printf ("AT*EASM=%s\r", mymenu->mainmenu);
	v24Puts (port, command);
	depth = 1;
      }
      if (strstr (buffer, "EAMI:")) {
	callstr = g_strdup (buffer + 7);
	call = atoi (callstr);
	//      g_print ("depth:%d, call:%d, dpth[call]:%d, nodetype:%d\n",
	//	       depth, call, mymenu->dpth[call], mymenu->nodetype[call][depth]);
	if (call) {
	  if (mymenu->nodetype[call][depth] == 1) {
	    args = g_strsplit (mymenu->column2[call][depth], " ", 100);
	    k = 0;
	    while (args[k] != NULL) {
	      argv[k] = g_strdup (args[k]);
	      k++;
	    }
	    argv[k] = (char *) NULL;
    	    runcmd (argv[0], argv);
	  }
	  if (depth == 1)
	    depth = mymenu->dpth[call];
	  //	else
	  //	  depth += dpth[call];
	  if (depth > 1) {
	    if (i == 0)
	      i = call;
	    if (mymenu->submenu[i] != NULL)
	      command = g_strdup_printf ("AT*EASM=%s\r", mymenu->submenu[i]);
	    else {
	      command = g_strdup_printf ("AT*EASM=%s\r", mymenu->mainmenu);
	      depth = 1;
	      i = 0;
	    }
	  }
	  else if (depth == 1)
	    command = g_strdup_printf ("AT*EASM=%s\r", mymenu->mainmenu);
	  v24Puts (port, command);
	}
	else {
	  if (depth != 1) {
	    command = g_strdup_printf ("AT*EASM=%s\r", mymenu->mainmenu);
	    v24Puts (port, command);
	    depth = 1;
	    i = 0;
	  }
	}
      }
    }
  }
  else {
    gtk_object_set_data (GTK_OBJECT (ctree), "pid", (gpointer) pid);
    signal (SIGCHLD, mySignalHandler);
  }
  g_free (mymenu);
  if (command != NULL)
    g_free (command);
  if (tmp != NULL)
    g_free (tmp);
  if (callstr != NULL)
    g_free (callstr);
  if (args != NULL)
    g_strfreev (args);
}
