/*
  tsemgr is a gtk application to manage a sonyericsson t68 mobile phone
  Copyright (C) 2003 Sebastian Sareyko

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/


#include "config.h"
#include <string.h>
#include <stdio.h>
#include <ezV24/ezV24.h>
#include "sms.h"
#include "interface.h"
#include "gsm.h"
#include "misc.h"

/* 
   sms_data_7bit_decode was taken from the scmxx-0.6.3.5
   (which was made by Hendrik Sattler) source, and
   then modified to my needs!
   go get it at: http://www.hendrik-sattler.de/scmxx/
*/
gchar* sms_data_7bit_decode (gchar* input, gint encoded_size) {
  unsigned char userdata[281];
  unsigned char decoded[161];
  gint i, first, end, offset, counter;

  if (!encoded_size || input == NULL) {
    return NULL;
  }

  counter = 0;
  for (i = 0; i < strlen (input); i += 2) {
    userdata[counter++] = hexstr2int (input + i, 2);
  }
  offset = -1;
  counter = 0;
  for (i = 0; i < encoded_size; i++) {
    if (i % 8) {
      first = userdata[i - 1 - offset];
      first >>= 8 - (i % 8);
      end = userdata[i - offset];
      end <<= (i % 8);
      *(decoded + counter) = (first | end) & 127;
    } else {
      ++offset;
      *(decoded + counter) = userdata[i - offset] & 127;
    }
    if (*(decoded + counter) == 0) { //special char @
      *(decoded + counter) = 128;
    }
    ++counter;
  }
  decoded[counter] = '\0';
  return convert_from_gsm (decoded);
}

gchar* sms_filldata_7bit_decode (gchar* input, gint encoded_size) {
  unsigned char userdata[281];
  unsigned char decoded[161];
  gint i, first, tmp, last, counter, ls;

  if (input == NULL || encoded_size == 0)
    return NULL;

  counter = 0;
  for (i = 0; i < strlen (input); i += 2) {
    userdata[counter++] = hexstr2int (input + i, 2);
  }
  first = 0;
  last = 0;
  ls = 6;
  counter = 1;
  tmp = 1;
  for (i = encoded_size; i > 0;) {
    --i;
    first = userdata[i];
    first >>= counter;
    *(decoded + i) = (first | last) & 127;
    last = userdata[i] & tmp;
    last <<= ls;
    --ls;
    tmp = (tmp * 2) + 1;
    ++counter;
  }
  decoded[counter - 1] = '\0';
  return convert_from_gsm (decoded);
}

/* 
   sms_data_7bit_encode was taken from the scmxx-0.6.3.5
   (which was made by Hendrik Sattler) source, and
   then modified to my needs!
   go get it at: http://www.hendrik-sattler.de/scmxx/
*/
gchar* sms_data_7bit_encode (gchar* input, gint *encoded_size){
  unsigned char* temp;
  gint i, first, end, outsize;
  gchar* retval;

  if (input == NULL || encoded_size == NULL)
    return NULL;
  temp = convert_to_gsm (input);

  *encoded_size = strlen (temp);
  outsize = 0;
  retval = g_strdup ("");
  // 7bit-GSM octet encoding
  for (i = 0; i < strlen (temp); i++) {
    end = temp[i] & 127; // the character are 7bit
    end >>= (i % 8);
    first = temp[i+1] & 127;
    first <<= 7 - (i % 8);
    first &= 255; // we only need the last eight bits (AND mask)
    retval = g_strdup_printf ("%s%02X", retval, (first | end));
    ++outsize;
    if ((i % 8) == 6) {// every 8th char is already coded in (it really must be 6)
      i++;
    }
  }
  return retval;
}

recievedSMS* parseSMS (gchar* string) {
  gint i = 0;
  gint j, k;
  recievedSMS *toReturn;
  gint SMSClength;
  unsigned char SMSCtype;
  gchar *SMSCnumber;
  unsigned char firstOctet;
  gint addressLength;
  gint senderType;
  gchar *senderNumber;
  unsigned char protocol;
  unsigned char dataCodingScheme;
  gchar *timeStamp;
  gint messageLength;
  gchar *message, *tmp;
  gchar *reference;
  //  gchar *startOfText;
  gint iei = 999;
  gint iel;
  gint ie[256];
  gint udhLength;
  gint udh;
  gint al_odd = 0;
	
  SMSClength = hexstr2int (string, 2);
  i = SMSClength * 2 + 2;
  udh = hexstr2int (string + i,  2);
  i += 2;
  addressLength = hexstr2int (string + i, 2);
  i += 2;
  senderType = hexstr2int (string + i, 2);
  i += 2;
  if (addressLength & 1) {
    ++addressLength;
    al_odd = 1;
  }
  if ((senderType >> 4) == 13) {  // Alphanumeric Format
    addressLength /= 2;
    senderNumber = sms_data_7bit_decode (string + i, addressLength);
    j = addressLength * 2;
  }
  else if ((senderType >> 4) == 9) {  // International Format
    senderNumber = g_malloc (sizeof (gchar) * (addressLength + 1));
    for (j = 0; j < addressLength; j += 2) {
      senderNumber[j] = string[i + j + 1];
      senderNumber[j + 1] = string[i + j];
    }
    if (al_odd)
      senderNumber[addressLength - 1] = '\0';
    else
      senderNumber[addressLength] = '\0';
    senderNumber = g_strdup_printf ("+%s", senderNumber);
  }
  else if ((senderType >> 4) == 10) {  // National Format
    senderNumber = g_malloc (sizeof (gchar) * (addressLength + 1));
    for (j = 0; j < addressLength; j += 2) {
      senderNumber[j] = string[i + j + 1];
      senderNumber[j + 1] = string[i + j];
    }
    if (al_odd)
      senderNumber[addressLength - 1] = '\0';
    else
      senderNumber[addressLength] = '\0';
  }
  else
    senderNumber = g_strdup ("Unknown Number Format");

  i += addressLength + 4;

  //  timeStamp = g_malloc (sizeof (gchar) * 12);
  timeStamp = g_strdup_printf ("%c%c.%c%c.%c%c, %c%c:%c%c.%c%c",
			       string[i+5], string[i+4],  //day
			       string[i+3], string[i+2],  //month
			       string[i+1], string[i],    //year
			       string[i+7], string[i+6],  //hour
			       string[i+9], string[i+8],  //minute
			       string[i+11], string[i+10]); //second

  i += 14;
  messageLength = hexstr2int (string + i, 2);
  i += 2;
  toReturn = g_malloc (sizeof (recievedSMS));
  toReturn->udh = 0;
  toReturn->concat = 0;
  toReturn->concat_reference = 0;
  toReturn->concat_max = 0;
  toReturn->concat_cur = 0;
  toReturn->pda = 0;
  toReturn->la = 0;
  toReturn->sa = 0;
  toReturn->lp = 0;
  toReturn->sp = 0;
  toReturn->vp = 0;
  toReturn->pds = 0;
  toReturn->uds = 0;
  if (udh & 0x40) {
    //    toReturn->udhfields = g_malloc (sizeof (udhdata));
    toReturn->udh = 1;
    udhLength = hexstr2int (string + i, 2);
    //    g_print ("\nudh found, length, messageLength: %d %d\n", udhLength, messageLength);
    //    messageLength = messageLength - udhLength - 4;
    messageLength = messageLength - (1 + ((udhLength * 8) + (udhLength * 8) % 7) / 7);
    //    g_print ("\nudh found, length, messageLength: %d %d\n", udhLength, messageLength);
    for (j = 0; j < (udhLength * 2) - 2;) {
      iei = hexstr2int (string + i + j + 2, 2);
      iel = hexstr2int (string + i + j + 4, 2);
      //      g_print ("udh found, length: %d, type: %d\n", iel, iei);
      for (k = 0; k < iel; k++) {
	ie[k] = hexstr2int (string + i + 6 + j + (k * 2), 2);
	//	g_print ("ie%d:k%d:iel%d\n", ie[k], k, iel);
      }
      switch (iei) {
      case 0: /* Concatenated short message, 8-bit reference number */
	toReturn->concat = 1;
	toReturn->concat_reference = ie[0];
	toReturn->concat_max = ie[1];
	toReturn->concat_cur = ie[2];
	break;
      case 8: /* Concatenated short message, 16-bit reference number */
	toReturn->concat = 1;
	reference = g_strdup_printf ("%02X%02X", ie[0], ie[1]);
	toReturn->concat_reference = hexstr2int (reference, 4);
	toReturn->concat_max = ie[2];
	toReturn->concat_cur = ie[3];
	break;
      case 10: /* Text Formatting */
	break;
      case 11: /* Predefined Sound */
	toReturn->pds_pos[toReturn->pds] = ie[0];
	++toReturn->pds;
	break;
      case 12: /* User Defined Sound */
	toReturn->uds_pos[toReturn->uds] = ie[0];
	++toReturn->uds;
	break;
      case 13: /* Predefined Animation */
	toReturn->pda_pos[toReturn->pda] = ie[0];
	toReturn->pda_nr[toReturn->pda] = ie[1];
	++toReturn->pda;
	break;
      case 14: /* Large Animation (16*16 times 4 = 32*4 =128 bytes) */
	toReturn->la_pos[toReturn->la] = ie[0];
	++toReturn->la;
	break;
      case 15: /* Small Animation (8*8 times 4 = 8*4 =32 bytes) */
	toReturn->sa_pos[toReturn->sa] = ie[0];
	++toReturn->sa;
	break;
      case 16: /* Large Picture (32*32 = 128 bytes) */
	toReturn->lp_pos[toReturn->lp] = ie[0];
	++toReturn->lp;
	break;
      case 17: /* Small Picture (16*16 = 32 bytes) */
	toReturn->sp_pos[toReturn->sp] = ie[0];
	++toReturn->sp;
	break;
      case 18: /* Variable Picture */
	toReturn->vp_pos[toReturn->vp] = ie[0];
	++toReturn->vp;
	break;
      case 999: /* nothing found */
	break;
      default:
	break;
      }
      j += (iel * 2) + 4;
    }
    i += (udhLength * 2) + 2;
    //    i += ((1 + (((udhLength * 8) + ((udhLength * 8) % 7)) / 7)) * 2) - 2;
  }
  if ((udh & 0x40 ) && ((1120 - ((udhLength + 1) * 8)) % 7)) {
    for (j = 0; ((1120 - ((udhLength + j + 1) * 8)) % 7) != 0; j++) {
      messageLength--;
      i += 2;
    }
  }
  if (messageLength + j > 0) {
    //    g_print ("all:%s\ndecode:%s i:%d strlen:%d\n", string, string + i, i,
    //	     strlen (string + i));
    if (messageLength > 0) {
      //      message = g_malloc (sizeof (gchar) * messageLength + 1);
      message = sms_data_7bit_decode (string + i, messageLength);
    }
    else
      message = g_strdup ("");
    if ((udh & 0x40) && ((1120 - ((udhLength + 1) * 8)) % 7)) {
      //      g_print ("%s\n", string);
      //      g_print ("----> %s %d\n", string + i - (j * 2), j);
      //      g_print ("%s\n", message);
      tmp = sms_filldata_7bit_decode (string + i - (j * 2), j);
      message = g_strdup_printf ("%s%s", tmp, message);
      messageLength += j;
    }
  }
  else {
    //    message = g_malloc (sizeof (gchar) * 23);
    messageLength = 0;
    message = g_strdup ("");
  }

  toReturn->SMSClength = SMSClength;
  toReturn->SMSCtype = SMSCtype;
  toReturn->SMSCnumber = SMSCnumber;
  toReturn->firstOctet = firstOctet;
  toReturn->addressLength = addressLength;
  toReturn->senderType = senderType;
  toReturn->senderNumber = senderNumber;
  toReturn->protocol = protocol;
  toReturn->dataCodingScheme = dataCodingScheme;
  toReturn->timeStamp = timeStamp;
  toReturn->messageLength = messageLength;
  toReturn->message = message;

  return toReturn;
}

GSList* listSMS (gint memtype, gint type, GtkWidget *progress_bar) {
  GtkWidget *label;
  GSList *toReturn = NULL;
  gchar buffer[512];
  gchar **indexstr;
  gchar *command;
  gchar *status;
  gint stop = 0;
  gint failed = 0;
  gint index;
  gfloat i = 1.0;
  v24_port_t *port = NULL;

  port = v24OpenPort (config.device, V24_RTS_CTS);
  if (port != NULL) {
    switch (memtype) {
    case SIM:  v24Puts(port, "AT+CPMS=\"SM\"\r");
      break;
    case ME:   v24Puts(port, "AT+CPMS=\"ME\"\r");
      break;
    }
    while (!stop) {
      v24Gets (port, buffer, 512);
      if (strcasecmp (buffer, "OK\r\n") == 0)
	stop = 1;
      else if (strstr (buffer, "ERROR")) {
	toReturn = NULL;
	failed = 1;
	stop = 1;
      }
    }
    stop = 0;
    command = g_strdup_printf ("AT+CMGL=%d\r", type);
  
    v24Puts (port, command);
    v24Gets (port, buffer, 512);

    if (!failed) {
      label = gtk_object_get_data (GTK_OBJECT (progress_bar), "statuslabel");
      while (!stop) {
	v24Gets (port, buffer, 512);
	//	g_print ("%s\n", buffer);
	if (strstr (buffer, "ERROR")) {
	  toReturn = NULL;
	  stop = 1;
	}
	else if (strstr (buffer, "+CMGL: ")) {
	  //      tmp = g_strdup (buffer);
	  indexstr = g_strsplit (buffer + 6, ",", 3);
	  index = atoi (indexstr[0]);
	  indexstr[1] = g_strdup_printf ("%02X", index);
	  //	  g_print ("%s\n", indexstr[1]);
	  v24Gets (port, buffer, 512);
	  gtk_progress_set_value ((GtkProgress *) progress_bar, i);
	  status = g_strdup_printf ("message #%d", (gint) i);
	  gtk_label_set_text (GTK_LABEL (label), status);
	  while (gtk_events_pending())
	    gtk_main_iteration();
	  i += 1;
	  //	  g_print ("%s\n", buffer);
	  //      if ((buffer[0] == '0') && (buffer[1] == '7'))
	  toReturn = g_slist_append (toReturn,
				     g_strdup_printf ("%s%s", indexstr[1], buffer));
	  g_free (status);
	  g_strfreev (indexstr);
	}
	if (strcasecmp (buffer, "OK\r\n") == 0)
	  stop = 1;
      }
    }
    v24ClosePort (port);
    //  gtk_widget_destroy (dialog);
    g_free (command);
  }
  else
    errorDialog ("unable to open device!\nplease check if your phone is connected\nand if you specified the correct device!");

  return toReturn;
}

gint encode_and_sendSMS (gchar *number, gchar *message, gint store) {
  gchar *tosend, *encoded, *temp, *messageflags, *tmpmsg;
  gint len, i, cur, total;
  gint j = 0;
  gint toReturn = 1;

  total = strlen (message) / 160;
  if (strlen (message) % 160)
    total++;
  for (cur = 1; cur <= total; cur++) {
    if (strlen (message) > 160)
      messageflags = g_strdup_printf ("41");
    else
      messageflags = g_strdup_printf ("01");
    tosend = g_strdup_printf ("00"); // SMSC length
    tosend = g_strdup_printf ("%s%s", tosend, messageflags); // message flags
    tosend = g_strdup_printf ("%s00", tosend); // message reference number
    if (number[0] == '+')
      tosend = g_strdup_printf ("%s%02X", tosend, strlen (number) - 1); // adress length
    else
      tosend = g_strdup_printf ("%s%02X", tosend, strlen (number)); // adress length
    //  if (number[0] == '4')
    //    tosend = g_strdup_printf ("%s91", tosend); // adress type
    if (number[0] == '+') {
      tosend = g_strdup_printf ("%s91", tosend); // adress type
      number = g_strdup_printf ("%s", number + 1);
    }
    else
      tosend = g_strdup_printf ("%s81", tosend); // adress type
    if (strlen (number) % 2 != 0)
      number = g_strdup_printf ("%sF", number);
    temp = g_strdup_printf ("%c%c", number[1], number[0]);
    for (i = 2; i < strlen (number); i += 2)
      temp = g_strdup_printf ("%s%c%c", temp, number[i + 1], number[i]);
    tosend = g_strdup_printf ("%s%s", tosend, temp); // adress number
    tosend = g_strdup_printf ("%s00", tosend); // protocol identifier
    tosend = g_strdup_printf ("%s00", tosend); // data coding scheme
    if (strlen (message + j) >= 160)
      tmpmsg = g_strndup (message + j, 160);
    else
      tmpmsg = g_strndup (message + j, strlen (message + j));
    g_print ("%d %d\n%s\n", total, cur, tmpmsg);
    encoded = sms_data_7bit_encode (tmpmsg, &len);
    if (strlen (message) > 160) {
      tosend = g_strdup_printf ("%s%02X", tosend, len + 6); // ud length + udh lenght
      tosend = g_strdup_printf ("%s%02X", tosend, 6); // udh length
      tosend = g_strdup_printf ("%s%02X", tosend, 8); // iei
      tosend = g_strdup_printf ("%s%02X", tosend, 4); // ie length
      tosend = g_strdup_printf ("%s%02X", tosend, 0); // ie0 (16bit reference part I)
      tosend = g_strdup_printf ("%s%02X", tosend, 23); // ie1 (16bit reference part II)
      tosend = g_strdup_printf ("%s%02X", tosend, total); // ie2 (total)
      tosend = g_strdup_printf ("%s%02X", tosend, cur); // ie3 (current)
    }
    else
      tosend = g_strdup_printf ("%s%02X", tosend, len); // ud length
    tosend = g_strdup_printf ("%s%s", tosend, encoded); // pdu
    g_print ("%s\n%d\n", tosend, (strlen (tosend) / 2) - 1);
    if (sendencodedSMS (tosend, store) == 0)
      toReturn = 0;
    g_free (tosend);
    g_free (temp);
    g_free (tmpmsg);
    g_free (encoded);
    j += 160;
  }

  return toReturn;
}

gint sendencodedSMS (gchar *data, gint store) {
  GtkWidget *dialog;
  gchar *command;
  gchar buffer[512];
  gint stop = 0;
  gint toReturn = 1;
  v24_port_t *port = NULL;

  dialog = messageDialog ("sending message...\nplease wait", 0);
  while (gtk_events_pending())
    gtk_main_iteration();
  port = v24OpenPort (config.device, V24_RTS_CTS);
  if (port != NULL) {
    if (store == 0)
      command = g_strdup_printf ("AT+CMGS=%d\r", (strlen (data) / 2) - 1);
    else 
      command = g_strdup_printf ("AT+CMGW=%d\r", (strlen (data) / 2) - 1);
    v24Puts (port, command);
    while (!stop) {
      while (gtk_events_pending())
	gtk_main_iteration();
      v24Gets (port, buffer, 512);
      if (strstr (buffer, ">"))
	stop = 1;
      else if (strstr (buffer, "ERROR")) {
	stop = 1;
	toReturn = 0;
      }
    }
    command = g_strdup_printf ("%s", data);
    v24Puts (port, command);
    command = g_strdup_printf ("\x1a");
    v24Puts (port, command);
    v24Gets (port, buffer, 512);
    v24Gets (port, buffer, 512);
    if (strstr (buffer, "ERROR"))
      toReturn = 0;
    v24ClosePort (port);
    g_free (command);
  }
  else {
    toReturn = 0;
    errorDialog ("unable to open device!\nplease check if your phone is connected\nand if you specified the correct device!");
  }

  gtk_widget_destroy (dialog);
  return toReturn;
}

gint deleteSMS (gint index, gint mem) {
  GtkWidget *dialog;
  gchar *command;
  gchar buffer[512];
  gint stop = 0;
  gint toReturn = 1;
  v24_port_t *port = NULL;

  dialog = messageDialog ("removing message from phone...\nplease wait", 0);
  while (gtk_events_pending())
    gtk_main_iteration();
  port = v24OpenPort (config.device, V24_RTS_CTS);
  if (port != NULL) {
    switch (mem) {
    case SIM:  v24Puts(port, "AT+CPMS=\"SM\"\r");
      break;
    case ME:   v24Puts(port, "AT+CPMS=\"ME\"\r");
      break;
    }
    while (!stop) {
      v24Gets (port, buffer, 512);
      if (strstr (buffer, "OK"))
	stop = 1;
      else if (strstr (buffer, "ERROR")) {
	stop = 1;
	toReturn = 0;
      }
    }
    stop = 0;
    if (toReturn) {
      g_print ("%d\n", index);
      command = g_strdup_printf ("AT+CMGD=%d\r", index);
      v24Puts (port, command);
      while (!stop) {
	while (gtk_events_pending())
	  gtk_main_iteration();
	v24Gets (port, buffer, 512);
	g_print ("%s\n", buffer);
	if (strstr (buffer, "OK"))
	  stop = 1;
	else if (strstr (buffer, "ERROR")) {
	  stop = 1;
	  toReturn = 0;
	}
      }
    }
    v24ClosePort (port);
    g_free (command);
    gtk_widget_destroy (dialog);
  }
  else {
    toReturn = 0;
    gtk_widget_destroy (dialog);
    errorDialog ("unable to open device!\nplease check if your phone is connected\nand if you specified the correct device!");
  }

  return toReturn;
}
